import voluptuous as vol
from .const import ATTR_ATTRIBUTES as ATTR_ATTRIBUTES, ATTR_BATTERY as ATTR_BATTERY, ATTR_CONSIDER_HOME as ATTR_CONSIDER_HOME, ATTR_DEV_ID as ATTR_DEV_ID, ATTR_GPS as ATTR_GPS, ATTR_HOST_NAME as ATTR_HOST_NAME, ATTR_LOCATION_NAME as ATTR_LOCATION_NAME, ATTR_MAC as ATTR_MAC, ATTR_SOURCE_TYPE as ATTR_SOURCE_TYPE, CONF_CONSIDER_HOME as CONF_CONSIDER_HOME, CONF_NEW_DEVICE_DEFAULTS as CONF_NEW_DEVICE_DEFAULTS, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, CONF_TRACK_NEW as CONF_TRACK_NEW, DEFAULT_CONSIDER_HOME as DEFAULT_CONSIDER_HOME, DEFAULT_TRACK_NEW as DEFAULT_TRACK_NEW, DOMAIN as DOMAIN, LOGGER as LOGGER, PLATFORM_TYPE_LEGACY as PLATFORM_TYPE_LEGACY, SCAN_INTERVAL as SCAN_INTERVAL, SOURCE_TYPE_BLUETOOTH as SOURCE_TYPE_BLUETOOTH, SOURCE_TYPE_BLUETOOTH_LE as SOURCE_TYPE_BLUETOOTH_LE, SOURCE_TYPE_GPS as SOURCE_TYPE_GPS, SOURCE_TYPE_ROUTER as SOURCE_TYPE_ROUTER
from collections.abc import Coroutine, Sequence
from datetime import timedelta
from homeassistant import util as util
from homeassistant.components import zone as zone
from homeassistant.config import async_log_exception as async_log_exception, load_yaml_config_file as load_yaml_config_file
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_GPS_ACCURACY as ATTR_GPS_ACCURACY, ATTR_ICON as ATTR_ICON, ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE, ATTR_NAME as ATTR_NAME, CONF_ICON as CONF_ICON, CONF_MAC as CONF_MAC, CONF_NAME as CONF_NAME, DEVICE_DEFAULT_NAME as DEVICE_DEFAULT_NAME, STATE_HOME as STATE_HOME, STATE_NOT_HOME as STATE_NOT_HOME
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import config_per_platform as config_per_platform, discovery as discovery
from homeassistant.helpers.entity_registry import async_get_registry as async_get_registry
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval, async_track_utc_time_change as async_track_utc_time_change
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.typing import ConfigType as ConfigType, GPSType as GPSType, StateType as StateType
from homeassistant.setup import async_prepare_setup_platform as async_prepare_setup_platform, async_start_setup as async_start_setup
from homeassistant.util import dt as dt_util
from homeassistant.util.yaml import dump as dump
from types import ModuleType
from typing import Any, Callable, Final

SERVICE_SEE: Final[str]
SOURCE_TYPES: Final[tuple[str, ...]]
NEW_DEVICE_DEFAULTS_SCHEMA: Any
PLATFORM_SCHEMA: Final[Any]
PLATFORM_SCHEMA_BASE: Final[vol.Schema]
SERVICE_SEE_PAYLOAD_SCHEMA: Final[vol.Schema]
YAML_DEVICES: Final[str]
EVENT_NEW_DEVICE: Final[str]

def see(hass: HomeAssistant, mac: Union[str, None] = ..., dev_id: Union[str, None] = ..., host_name: Union[str, None] = ..., location_name: Union[str, None] = ..., gps: Union[GPSType, None] = ..., gps_accuracy: Union[int, None] = ..., battery: Union[int, None] = ..., attributes: Union[dict, None] = ...) -> None: ...
async def async_setup_integration(hass: HomeAssistant, config: ConfigType) -> None: ...

class DeviceTrackerPlatform:
    LEGACY_SETUP: Final[tuple[str, ...]]
    name: str
    platform: ModuleType
    config: dict
    @property
    def type(self) -> Union[str, None]: ...
    async def async_setup_legacy(self, hass: HomeAssistant, tracker: DeviceTracker, discovery_info: Union[dict[str, Any], None] = ...) -> None: ...
    def __init__(self, name, platform, config) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

async def async_extract_config(hass: HomeAssistant, config: ConfigType) -> list[DeviceTrackerPlatform]: ...
async def async_create_platform_type(hass: HomeAssistant, config: ConfigType, p_type: str, p_config: dict) -> Union[DeviceTrackerPlatform, None]: ...
def async_setup_scanner_platform(hass: HomeAssistant, config: ConfigType, scanner: DeviceScanner, async_see_device: Callable[..., Coroutine[None, None, None]], platform: str) -> None: ...
async def get_tracker(hass: HomeAssistant, config: ConfigType) -> DeviceTracker: ...

class DeviceTracker:
    hass: Any
    devices: Any
    mac_to_dev: Any
    consider_home: Any
    track_new: Any
    defaults: Any
    _is_updating: Any
    def __init__(self, hass: HomeAssistant, consider_home: timedelta, track_new: bool, defaults: dict[str, Any], devices: Sequence[Device]) -> None: ...
    def see(self, mac: Union[str, None] = ..., dev_id: Union[str, None] = ..., host_name: Union[str, None] = ..., location_name: Union[str, None] = ..., gps: Union[GPSType, None] = ..., gps_accuracy: Union[int, None] = ..., battery: Union[int, None] = ..., attributes: Union[dict, None] = ..., source_type: str = ..., picture: Union[str, None] = ..., icon: Union[str, None] = ..., consider_home: Union[timedelta, None] = ...) -> None: ...
    async def async_see(self, mac: Union[str, None] = ..., dev_id: Union[str, None] = ..., host_name: Union[str, None] = ..., location_name: Union[str, None] = ..., gps: Union[GPSType, None] = ..., gps_accuracy: Union[int, None] = ..., battery: Union[int, None] = ..., attributes: Union[dict, None] = ..., source_type: str = ..., picture: Union[str, None] = ..., icon: Union[str, None] = ..., consider_home: Union[timedelta, None] = ...) -> None: ...
    async def async_update_config(self, path: str, dev_id: str, device: Device) -> None: ...
    def async_update_stale(self, now: dt_util.dt.datetime) -> None: ...
    async def async_setup_tracked_device(self) -> None: ...

class Device(RestoreEntity):
    host_name: Union[str, None]
    location_name: Union[str, None]
    gps: Union[GPSType, None]
    gps_accuracy: int
    last_seen: Union[dt_util.dt.datetime, None]
    battery: Union[int, None]
    attributes: Union[dict, None]
    last_update_home: bool
    _state: str
    hass: Any
    entity_id: Any
    consider_home: Any
    dev_id: Any
    mac: Any
    track: Any
    config_name: Any
    config_picture: Any
    _icon: Any
    source_type: Any
    _attributes: Any
    def __init__(self, hass: HomeAssistant, consider_home: timedelta, track: bool, dev_id: str, mac: Union[str, None], name: Union[str, None] = ..., picture: Union[str, None] = ..., gravatar: Union[str, None] = ..., icon: Union[str, None] = ...) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def state(self) -> str: ...
    @property
    def entity_picture(self) -> Union[str, None]: ...
    @property
    def state_attributes(self) -> dict[str, StateType]: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...
    @property
    def icon(self) -> Union[str, None]: ...
    async def async_seen(self, host_name: Union[str, None] = ..., location_name: Union[str, None] = ..., gps: Union[GPSType, None] = ..., gps_accuracy: Union[int, None] = ..., battery: Union[int, None] = ..., attributes: Union[dict[str, Any], None] = ..., source_type: str = ..., consider_home: Union[timedelta, None] = ...) -> None: ...
    def stale(self, now: Union[dt_util.dt.datetime, None] = ...) -> bool: ...
    def mark_stale(self) -> None: ...
    async def async_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...

class DeviceScanner:
    hass: Union[HomeAssistant, None]
    def scan_devices(self) -> list[str]: ...
    async def async_scan_devices(self) -> list[str]: ...
    def get_device_name(self, device: str) -> Union[str, None]: ...
    async def async_get_device_name(self, device: str) -> Union[str, None]: ...
    def get_extra_attributes(self, device: str) -> dict: ...
    async def async_get_extra_attributes(self, device: str) -> dict: ...

async def async_load_config(path: str, hass: HomeAssistant, consider_home: timedelta) -> list[Device]: ...
def update_config(path: str, dev_id: str, device: Device) -> None: ...
def get_gravatar_for_email(email: str) -> str: ...



import voluptuous as vol
from . import ATTR_CODE_ARM_REQUIRED as ATTR_CODE_ARM_REQUIRED, DOMAIN as DOMAIN
from .const import SUPPORT_ALARM_ARM_AWAY as SUPPORT_ALARM_ARM_AWAY, SUPPORT_ALARM_ARM_HOME as SUPPORT_ALARM_ARM_HOME, SUPPORT_ALARM_ARM_NIGHT as SUPPORT_ALARM_ARM_NIGHT, SUPPORT_ALARM_ARM_VACATION as SUPPORT_ALARM_ARM_VACATION, SUPPORT_ALARM_TRIGGER as SUPPORT_ALARM_TRIGGER
from homeassistant.const import ATTR_CODE as ATTR_CODE, ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_CODE as CONF_CODE, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE, SERVICE_ALARM_ARM_AWAY as SERVICE_ALARM_ARM_AWAY, SERVICE_ALARM_ARM_HOME as SERVICE_ALARM_ARM_HOME, SERVICE_ALARM_ARM_NIGHT as SERVICE_ALARM_ARM_NIGHT, SERVICE_ALARM_ARM_VACATION as SERVICE_ALARM_ARM_VACATION, SERVICE_ALARM_DISARM as SERVICE_ALARM_DISARM, SERVICE_ALARM_TRIGGER as SERVICE_ALARM_TRIGGER
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.entity import get_supported_features as get_supported_features
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

ACTION_TYPES: Final[set[str]]
ACTION_SCHEMA: Final[Any]

async def async_get_actions(hass: HomeAssistant, device_id: str) -> list[dict[str, str]]: ...
async def async_call_action_from_config(hass: HomeAssistant, config: ConfigType, variables: dict, context: Union[Context, None]) -> None: ...
async def async_get_action_capabilities(hass: HomeAssistant, config: ConfigType) -> dict[str, vol.Schema]: ...



from . import DATA_UPDATED as DATA_UPDATED
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.const import DATA_RATE_MEGABITS_PER_SECOND as DATA_RATE_MEGABITS_PER_SECOND
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any

ICON: str

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class SpeedtestSensor(RestoreEntity, SensorEntity):
    _attr_name: str
    _attr_unit_of_measurement: Any
    _attr_icon: Any
    _attr_should_poll: bool
    _attr_state: Any
    _speedtest_data: Any
    def __init__(self, speedtest_data: dict[str, Any]) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def update(self) -> None: ...
    def _schedule_immediate_update(self) -> None: ...



from .const import DOMAIN as DOMAIN, PLATFORMS as PLATFORMS
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from typing import Any

CONFIG_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from .const import DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY
from .helpers import on_unload as on_unload
from datetime import datetime
from homeassistant.components import mysensors as mysensors
from homeassistant.components.sensor import DOMAIN as DOMAIN, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONDUCTIVITY as CONDUCTIVITY, DEGREE as DEGREE, DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_ILLUMINANCE as DEVICE_CLASS_ILLUMINANCE, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_VOLTAGE as DEVICE_CLASS_VOLTAGE, ELECTRIC_CURRENT_AMPERE as ELECTRIC_CURRENT_AMPERE, ELECTRIC_POTENTIAL_MILLIVOLT as ELECTRIC_POTENTIAL_MILLIVOLT, ELECTRIC_POTENTIAL_VOLT as ELECTRIC_POTENTIAL_VOLT, ENERGY_KILO_WATT_HOUR as ENERGY_KILO_WATT_HOUR, FREQUENCY_HERTZ as FREQUENCY_HERTZ, LENGTH_METERS as LENGTH_METERS, LIGHT_LUX as LIGHT_LUX, MASS_KILOGRAMS as MASS_KILOGRAMS, PERCENTAGE as PERCENTAGE, POWER_VOLT_AMPERE as POWER_VOLT_AMPERE, POWER_WATT as POWER_WATT, SOUND_PRESSURE_DB as SOUND_PRESSURE_DB, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, VOLUME_CUBIC_METERS as VOLUME_CUBIC_METERS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.dt import utc_from_timestamp as utc_from_timestamp

SENSORS: dict[str, Union[list[Union[str, None]], dict[str, list[Union[str, None]]]]]

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class MySensorsSensor(mysensors.device.MySensorsEntity, SensorEntity):
    @property
    def force_update(self) -> bool: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def icon(self) -> Union[str, None]: ...
    @property
    def last_reset(self) -> Union[datetime, None]: ...
    @property
    def state_class(self) -> Union[str, None]: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...
    def _get_sensor_type(self) -> list[Union[str, None]]: ...



from . import SimpliSafe as SimpliSafe, SimpliSafeEntity as SimpliSafeEntity
from .const import ATTR_ALARM_DURATION as ATTR_ALARM_DURATION, ATTR_ALARM_VOLUME as ATTR_ALARM_VOLUME, ATTR_CHIME_VOLUME as ATTR_CHIME_VOLUME, ATTR_ENTRY_DELAY_AWAY as ATTR_ENTRY_DELAY_AWAY, ATTR_ENTRY_DELAY_HOME as ATTR_ENTRY_DELAY_HOME, ATTR_EXIT_DELAY_AWAY as ATTR_EXIT_DELAY_AWAY, ATTR_EXIT_DELAY_HOME as ATTR_EXIT_DELAY_HOME, ATTR_LIGHT as ATTR_LIGHT, ATTR_VOICE_PROMPT_VOLUME as ATTR_VOICE_PROMPT_VOLUME, DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN, LOGGER as LOGGER, VOLUME_STRING_MAP as VOLUME_STRING_MAP
from homeassistant.components.alarm_control_panel import AlarmControlPanelEntity as AlarmControlPanelEntity, FORMAT_NUMBER as FORMAT_NUMBER, FORMAT_TEXT as FORMAT_TEXT
from homeassistant.components.alarm_control_panel.const import SUPPORT_ALARM_ARM_AWAY as SUPPORT_ALARM_ARM_AWAY, SUPPORT_ALARM_ARM_HOME as SUPPORT_ALARM_ARM_HOME
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_CODE as CONF_CODE, STATE_ALARM_ARMED_AWAY as STATE_ALARM_ARMED_AWAY, STATE_ALARM_ARMED_HOME as STATE_ALARM_ARMED_HOME, STATE_ALARM_ARMING as STATE_ALARM_ARMING, STATE_ALARM_DISARMED as STATE_ALARM_DISARMED, STATE_ALARM_TRIGGERED as STATE_ALARM_TRIGGERED
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from simplipy.system.v2 import SystemV2 as SystemV2
from simplipy.system.v3 import SystemV3
from typing import Any

ATTR_BATTERY_BACKUP_POWER_LEVEL: str
ATTR_GSM_STRENGTH: str
ATTR_PIN_NAME: str
ATTR_RF_JAMMING: str
ATTR_WALL_POWER_LEVEL: str
ATTR_WIFI_STRENGTH: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanelEntity):
    _attr_code_format: Any
    _attr_supported_features: Any
    _last_event: Any
    _attr_state: Any
    def __init__(self, simplisafe: SimpliSafe, system: Union[SystemV2, SystemV3]) -> None: ...
    def _is_code_valid(self, code: Union[str, None], state: str) -> bool: ...
    async def async_alarm_disarm(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_home(self, code: Union[str, None] = ...) -> None: ...
    async def async_alarm_arm_away(self, code: Union[str, None] = ...) -> None: ...
    def async_update_from_rest_api(self) -> None: ...



import re
from homeassistant.const import CONF_DOMAINS as CONF_DOMAINS, CONF_ENTITIES as CONF_ENTITIES, CONF_EXCLUDE as CONF_EXCLUDE, CONF_INCLUDE as CONF_INCLUDE
from homeassistant.core import split_entity_id as split_entity_id
from typing import Any, Callable

CONF_INCLUDE_DOMAINS: str
CONF_INCLUDE_ENTITY_GLOBS: str
CONF_INCLUDE_ENTITIES: str
CONF_EXCLUDE_DOMAINS: str
CONF_EXCLUDE_ENTITY_GLOBS: str
CONF_EXCLUDE_ENTITIES: str
CONF_ENTITY_GLOBS: str

def convert_filter(config: dict[str, list[str]]) -> Callable[[str], bool]: ...

BASE_FILTER_SCHEMA: Any
FILTER_SCHEMA: Any

def convert_include_exclude_filter(config: dict[str, dict[str, list[str]]]) -> Callable[[str], bool]: ...

INCLUDE_EXCLUDE_FILTER_SCHEMA_INNER: Any
INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA: Any
INCLUDE_EXCLUDE_FILTER_SCHEMA: Any

def _glob_to_re(glob: str) -> re.Pattern[str]: ...
def _test_against_patterns(patterns: list[re.Pattern[str]], entity_id: str) -> bool: ...
def generate_filter(include_domains: list[str], include_entities: list[str], exclude_domains: list[str], exclude_entities: list[str], include_entity_globs: list[str] = ..., exclude_entity_globs: list[str] = ...) -> Callable[[str], bool]: ...



import aioshelly
from .const import BASIC_INPUTS_EVENTS_TYPES as BASIC_INPUTS_EVENTS_TYPES, CONF_COAP_PORT as CONF_COAP_PORT, DEFAULT_COAP_PORT as DEFAULT_COAP_PORT, DOMAIN as DOMAIN, SHBTN_INPUTS_EVENTS_TYPES as SHBTN_INPUTS_EVENTS_TYPES, SHBTN_MODELS as SHBTN_MODELS, SHIX3_1_INPUTS_EVENTS_TYPES as SHIX3_1_INPUTS_EVENTS_TYPES, UPTIME_DEVIATION as UPTIME_DEVIATION
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import singleton as singleton
from homeassistant.helpers.typing import EventType as EventType
from homeassistant.util.dt import utcnow as utcnow
from typing import Any, Final

_LOGGER: Final[Any]

async def async_remove_shelly_entity(hass: HomeAssistant, domain: str, unique_id: str) -> None: ...
def temperature_unit(block_info: dict[str, Any]) -> str: ...
def get_device_name(device: aioshelly.Device) -> str: ...
def get_number_of_channels(device: aioshelly.Device, block: aioshelly.Block) -> int: ...
def get_entity_name(device: aioshelly.Device, block: aioshelly.Block, description: Union[str, None] = ...) -> str: ...
def get_device_channel_name(device: aioshelly.Device, block: aioshelly.Block) -> str: ...
def is_momentary_input(settings: dict[str, Any], block: aioshelly.Block) -> bool: ...
def get_device_uptime(status: dict[str, Any], last_uptime: Union[str, None]) -> str: ...
def get_input_triggers(device: aioshelly.Device, block: aioshelly.Block) -> list[tuple[str, str]]: ...
async def get_coap_context(hass: HomeAssistant) -> aioshelly.COAP: ...
def get_device_sleep_period(settings: dict[str, Any]) -> int: ...



from .const import DATA_NEST as DATA_NEST, DOMAIN as DOMAIN, SIGNAL_NEST_UPDATE as SIGNAL_NEST_UPDATE
from homeassistant.components.climate import ClimateEntity as ClimateEntity, PLATFORM_SCHEMA as PLATFORM_SCHEMA
from homeassistant.components.climate.const import ATTR_TARGET_TEMP_HIGH as ATTR_TARGET_TEMP_HIGH, ATTR_TARGET_TEMP_LOW as ATTR_TARGET_TEMP_LOW, CURRENT_HVAC_COOL as CURRENT_HVAC_COOL, CURRENT_HVAC_HEAT as CURRENT_HVAC_HEAT, CURRENT_HVAC_IDLE as CURRENT_HVAC_IDLE, FAN_AUTO as FAN_AUTO, FAN_ON as FAN_ON, HVAC_MODE_AUTO as HVAC_MODE_AUTO, HVAC_MODE_COOL as HVAC_MODE_COOL, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, PRESET_ECO as PRESET_ECO, PRESET_NONE as PRESET_NONE, SUPPORT_FAN_MODE as SUPPORT_FAN_MODE, SUPPORT_PRESET_MODE as SUPPORT_PRESET_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE, SUPPORT_TARGET_TEMPERATURE_RANGE as SUPPORT_TARGET_TEMPERATURE_RANGE
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from typing import Any

_LOGGER: Any
NEST_MODE_HEAT_COOL: str
NEST_MODE_ECO: str
NEST_MODE_HEAT: str
NEST_MODE_COOL: str
NEST_MODE_OFF: str
MODE_HASS_TO_NEST: Any
MODE_NEST_TO_HASS: Any
ACTION_NEST_TO_HASS: Any
PRESET_AWAY_AND_ECO: str
PRESET_MODES: Any

def setup_platform(hass, config, add_entities, discovery_info: Any | None = ...) -> None: ...
async def async_setup_legacy_entry(hass, entry, async_add_entities) -> None: ...

class NestThermostat(ClimateEntity):
    _unit: Any
    structure: Any
    device: Any
    _fan_modes: Any
    _support_flags: Any
    _operation_list: Any
    _has_fan: Any
    _away: Any
    _location: Any
    _name: Any
    _humidity: Any
    _target_temperature: Any
    _temperature: Any
    _temperature_scale: Any
    _mode: Any
    _action: Any
    _fan: Any
    _eco_temperature: Any
    _is_locked: Any
    _locked_temperature: Any
    _min_temperature: Any
    _max_temperature: Any
    def __init__(self, structure, device, temp_unit) -> None: ...
    @property
    def should_poll(self): ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def supported_features(self): ...
    @property
    def unique_id(self): ...
    @property
    def device_info(self): ...
    @property
    def name(self): ...
    @property
    def temperature_unit(self): ...
    @property
    def current_temperature(self): ...
    @property
    def hvac_mode(self): ...
    @property
    def hvac_action(self): ...
    @property
    def target_temperature(self): ...
    @property
    def target_temperature_low(self): ...
    @property
    def target_temperature_high(self): ...
    def set_temperature(self, **kwargs) -> None: ...
    def set_hvac_mode(self, hvac_mode) -> None: ...
    @property
    def hvac_modes(self): ...
    @property
    def preset_mode(self): ...
    @property
    def preset_modes(self): ...
    def set_preset_mode(self, preset_mode) -> None: ...
    @property
    def fan_mode(self): ...
    @property
    def fan_modes(self): ...
    def set_fan_mode(self, fan_mode) -> None: ...
    @property
    def min_temp(self): ...
    @property
    def max_temp(self): ...
    def update(self) -> None: ...



from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntityDescription as SensorEntityDescription
from homeassistant.const import DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP, PERCENTAGE as PERCENTAGE
from typing import Any, Final

ATTR_BELT_UNIT_REMAINING_LIFE: Final[str]
ATTR_BLACK_DRUM_COUNTER: Final[str]
ATTR_BLACK_DRUM_REMAINING_LIFE: Final[str]
ATTR_BLACK_DRUM_REMAINING_PAGES: Final[str]
ATTR_BLACK_INK_REMAINING: Final[str]
ATTR_BLACK_TONER_REMAINING: Final[str]
ATTR_BW_COUNTER: Final[str]
ATTR_COLOR_COUNTER: Final[str]
ATTR_COUNTER: Final[str]
ATTR_CYAN_DRUM_COUNTER: Final[str]
ATTR_CYAN_DRUM_REMAINING_LIFE: Final[str]
ATTR_CYAN_DRUM_REMAINING_PAGES: Final[str]
ATTR_CYAN_INK_REMAINING: Final[str]
ATTR_CYAN_TONER_REMAINING: Final[str]
ATTR_DRUM_COUNTER: Final[str]
ATTR_DRUM_REMAINING_LIFE: Final[str]
ATTR_DRUM_REMAINING_PAGES: Final[str]
ATTR_DUPLEX_COUNTER: Final[str]
ATTR_FUSER_REMAINING_LIFE: Final[str]
ATTR_LASER_REMAINING_LIFE: Final[str]
ATTR_MAGENTA_DRUM_COUNTER: Final[str]
ATTR_MAGENTA_DRUM_REMAINING_LIFE: Final[str]
ATTR_MAGENTA_DRUM_REMAINING_PAGES: Final[str]
ATTR_MAGENTA_INK_REMAINING: Final[str]
ATTR_MAGENTA_TONER_REMAINING: Final[str]
ATTR_MANUFACTURER: Final[str]
ATTR_PAGE_COUNTER: Final[str]
ATTR_PF_KIT_1_REMAINING_LIFE: Final[str]
ATTR_PF_KIT_MP_REMAINING_LIFE: Final[str]
ATTR_REMAINING_PAGES: Final[str]
ATTR_STATUS: Final[str]
ATTR_UPTIME: Final[str]
ATTR_YELLOW_DRUM_COUNTER: Final[str]
ATTR_YELLOW_DRUM_REMAINING_LIFE: Final[str]
ATTR_YELLOW_DRUM_REMAINING_PAGES: Final[str]
ATTR_YELLOW_INK_REMAINING: Final[str]
ATTR_YELLOW_TONER_REMAINING: Final[str]
DATA_CONFIG_ENTRY: Final[str]
DOMAIN: Final[str]
UNIT_PAGES: Final[str]
PRINTER_TYPES: Final[Any]
SNMP: Final[str]
ATTRS_MAP: Final[dict[str, tuple[str, str]]]
SENSOR_TYPES: Final[tuple[SensorEntityDescription, ...]]



from homeassistant.components import switch as switch
from homeassistant.components.light import LightEntity as LightEntity, PLATFORM_SCHEMA as PLATFORM_SCHEMA
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_NAME as CONF_NAME, STATE_ON as STATE_ON, STATE_UNAVAILABLE as STATE_UNAVAILABLE
from homeassistant.core import HomeAssistant as HomeAssistant, State as State, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any

DEFAULT_NAME: str

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class LightSwitch(LightEntity):
    _name: Any
    _switch_entity_id: Any
    _unique_id: Any
    _switch_state: Any
    def __init__(self, name: str, switch_entity_id: str, unique_id: str) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def unique_id(self): ...
    async def async_turn_on(self, **kwargs) -> None: ...
    async def async_turn_off(self, **kwargs) -> None: ...
    async def async_added_to_hass(self) -> None: ...



from .const import DOMAIN as DOMAIN
from .knx_entity import KnxEntity as KnxEntity
from .schema import WeatherSchema as WeatherSchema
from homeassistant.components.weather import WeatherEntity as WeatherEntity
from homeassistant.const import CONF_NAME as CONF_NAME, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Weather as XknxWeather

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
def _create_weather(xknx: XKNX, config: ConfigType) -> XknxWeather: ...

class KNXWeather(KnxEntity, WeatherEntity):
    _device: XknxWeather
    _attr_temperature_unit: Any
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    @property
    def temperature(self) -> Union[float, None]: ...
    @property
    def pressure(self) -> Union[float, None]: ...
    @property
    def condition(self) -> str: ...
    @property
    def humidity(self) -> Union[float, None]: ...
    @property
    def wind_bearing(self) -> Union[int, None]: ...
    @property
    def wind_speed(self) -> Union[float, None]: ...



from . import HuaweiLteBaseEntity as HuaweiLteBaseEntity, Router as Router
from .const import CONF_TRACK_WIRED_CLIENTS as CONF_TRACK_WIRED_CLIENTS, DEFAULT_TRACK_WIRED_CLIENTS as DEFAULT_TRACK_WIRED_CLIENTS, DOMAIN as DOMAIN, KEY_LAN_HOST_INFO as KEY_LAN_HOST_INFO, KEY_WLAN_HOST_LIST as KEY_WLAN_HOST_LIST, UPDATE_SIGNAL as UPDATE_SIGNAL
from homeassistant.components.device_tracker.config_entry import ScannerEntity as ScannerEntity
from homeassistant.components.device_tracker.const import SOURCE_TYPE_ROUTER as SOURCE_TYPE_ROUTER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import entity_registry as entity_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any, Dict

_LOGGER: Any
_DEVICE_SCAN: Any
_HostType = Dict[str, Any]

def _get_hosts(router: Router, ignore_subscriptions: bool = ...) -> Union[list[_HostType], None]: ...
async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...
def _is_wireless(host: _HostType) -> bool: ...
def _is_connected(host: Union[_HostType, None]) -> bool: ...
def _is_us(host: _HostType) -> bool: ...
def async_add_new_entities(router: Router, async_add_entities: AddEntitiesCallback, tracked: set[str]) -> None: ...
def _better_snakecase(text: str) -> str: ...

class HuaweiLteScannerEntity(HuaweiLteBaseEntity, ScannerEntity):
    _mac_address: str
    _ip_address: Union[str, None]
    _is_connected: bool
    _hostname: Union[str, None]
    _extra_state_attributes: dict[str, Any]
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def source_type(self) -> str: ...
    @property
    def ip_address(self) -> Union[str, None]: ...
    @property
    def mac_address(self) -> str: ...
    @property
    def hostname(self) -> Union[str, None]: ...
    @property
    def is_connected(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...
    _available: bool
    async def async_update(self) -> None: ...
    def __init__(self, router, available, unsub_handlers, mac_address, ip_address, is_connected, hostname, extra_state_attributes) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...



from .const import DOMAIN as DOMAIN
from google_nest_sdm.device import Device as Device
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from typing import Any

DEVICE_TYPE_MAP: dict[str, str]

class NestDeviceInfo:
    device_brand: str
    _device: Any
    def __init__(self, device: Device) -> None: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    @property
    def device_name(self) -> str: ...
    @property
    def device_model(self) -> str: ...



from . import create_hyperion_client as create_hyperion_client
from .const import CONF_AUTH_ID as CONF_AUTH_ID, CONF_CREATE_TOKEN as CONF_CREATE_TOKEN, CONF_EFFECT_HIDE_LIST as CONF_EFFECT_HIDE_LIST, CONF_EFFECT_SHOW_LIST as CONF_EFFECT_SHOW_LIST, CONF_PRIORITY as CONF_PRIORITY, DEFAULT_ORIGIN as DEFAULT_ORIGIN, DEFAULT_PRIORITY as DEFAULT_PRIORITY, DOMAIN as DOMAIN
from homeassistant.components.ssdp import ATTR_SSDP_LOCATION as ATTR_SSDP_LOCATION, ATTR_UPNP_SERIAL as ATTR_UPNP_SERIAL
from homeassistant.config_entries import ConfigEntry as ConfigEntry, ConfigFlow as ConfigFlow, OptionsFlow as OptionsFlow, SOURCE_REAUTH as SOURCE_REAUTH
from homeassistant.const import CONF_BASE as CONF_BASE, CONF_HOST as CONF_HOST, CONF_ID as CONF_ID, CONF_PORT as CONF_PORT, CONF_SOURCE as CONF_SOURCE, CONF_TOKEN as CONF_TOKEN
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from hyperion import client
from typing import Any

_LOGGER: Any

class HyperionConfigFlow(ConfigFlow):
    VERSION: int
    _data: Any
    _request_token_task: Any
    _auth_id: Any
    _require_confirm: bool
    _port_ui: Any
    def __init__(self) -> None: ...
    def _create_client(self, raw_connection: bool = ...) -> client.HyperionClient: ...
    async def _advance_to_auth_step_if_necessary(self, hyperion_client: client.HyperionClient) -> FlowResult: ...
    async def async_step_reauth(self, config_data: dict[str, Any]) -> FlowResult: ...
    async def async_step_ssdp(self, discovery_info: dict[str, Any]) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def _cancel_request_token_task(self) -> None: ...
    async def _request_token_task_func(self, auth_id: str) -> None: ...
    def _get_hyperion_url(self) -> str: ...
    async def _can_login(self) -> Union[bool, None]: ...
    async def async_step_auth(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_create_token(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_create_token_external(self, auth_resp: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_create_token_success(self, _: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_create_token_fail(self, _: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> HyperionOptionsFlow: ...

class HyperionOptionsFlow(OptionsFlow):
    _config_entry: Any
    def __init__(self, config_entry: ConfigEntry) -> None: ...
    def _create_client(self) -> client.HyperionClient: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from collections.abc import Iterable
from typing import Any
from zwave_js_server.model.node import Node as ZwaveNode
from zwave_js_server.model.value import Value as ZwaveValue

class ZwaveValueID:
    property_: Union[str, int]
    command_class: int
    endpoint: Union[int, None]
    property_key: Union[str, int, None]

class BaseDiscoverySchemaDataTemplate:
    def resolve_data(self, value: ZwaveValue) -> dict[str, Any]: ...
    def values_to_watch(self, resolved_data: dict[str, Any]) -> Iterable[ZwaveValue]: ...
    def value_ids_to_watch(self, resolved_data: dict[str, Any]) -> set[str]: ...
    @staticmethod
    def _get_value_from_id(node: ZwaveNode, value_id_obj: ZwaveValueID) -> Union[ZwaveValue, None]: ...

class DynamicCurrentTempClimateDataTemplate(BaseDiscoverySchemaDataTemplate):
    lookup_table: dict[Union[str, int], ZwaveValueID]
    dependent_value: ZwaveValueID
    def resolve_data(self, value: ZwaveValue) -> dict[str, Any]: ...
    def values_to_watch(self, resolved_data: dict[str, Any]) -> Iterable[ZwaveValue]: ...
    @staticmethod
    def current_temperature_value(resolved_data: dict[str, Any]) -> Union[ZwaveValue, None]: ...



from .const import DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator, UpdateFailed as UpdateFailed
from typing import Any

PLATFORMS: Any
ATTR_SYSTEM_MODE: str
ATTR_SYSTEM_NAME: str
DEFAULT_ATTRIBUTION: str
DEFAULT_SCAN_INTERVAL: Any
CONFIG_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_register_new_bridge(hass: HomeAssistant, bridge: dict, entry: ConfigEntry) -> None: ...

class NotionEntity(CoordinatorEntity):
    _attr_device_class: Any
    _attr_device_info: Any
    _attr_extra_state_attributes: Any
    _attr_name: Any
    _attr_unique_id: Any
    _bridge_id: Any
    _sensor_id: Any
    _system_id: Any
    _task_id: Any
    def __init__(self, coordinator: DataUpdateCoordinator, task_id: str, sensor_id: str, bridge_id: str, system_id: str, name: str, device_class: str) -> None: ...
    @property
    def available(self) -> bool: ...
    async def _async_update_bridge_id(self) -> None: ...
    def _async_update_from_latest_data(self) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...



from . import AUTH_PROVIDERS as AUTH_PROVIDERS, AUTH_PROVIDER_SCHEMA as AUTH_PROVIDER_SCHEMA, AuthProvider as AuthProvider, LoginFlow as LoginFlow
from .. import InvalidAuthError as InvalidAuthError
from ..models import Credentials as Credentials, RefreshToken as RefreshToken, UserMeta as UserMeta
from collections.abc import Mapping
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from ipaddress import IPv4Address, IPv4Network, IPv6Address, IPv6Network
from typing import Any, Union

IPAddress = Union[IPv4Address, IPv6Address]
IPNetwork = Union[IPv4Network, IPv6Network]
CONF_TRUSTED_NETWORKS: str
CONF_TRUSTED_USERS: str
CONF_GROUP: str
CONF_ALLOW_BYPASS_LOGIN: str
CONFIG_SCHEMA: Any

class InvalidUserError(HomeAssistantError): ...

class TrustedNetworksAuthProvider(AuthProvider):
    DEFAULT_TITLE: str
    @property
    def trusted_networks(self) -> list[IPNetwork]: ...
    @property
    def trusted_users(self) -> dict[IPNetwork, Any]: ...
    @property
    def trusted_proxies(self) -> list[IPNetwork]: ...
    @property
    def support_mfa(self) -> bool: ...
    async def async_login_flow(self, context: Union[dict, None]) -> LoginFlow: ...
    async def async_get_or_create_credentials(self, flow_result: Mapping[str, str]) -> Credentials: ...
    async def async_user_meta_for_credentials(self, credentials: Credentials) -> UserMeta: ...
    def async_validate_access(self, ip_addr: IPAddress) -> None: ...
    def async_validate_refresh_token(self, refresh_token: RefreshToken, remote_ip: Union[str, None] = ...) -> None: ...

class TrustedNetworksLoginFlow(LoginFlow):
    _available_users: Any
    _ip_address: Any
    _allow_bypass_login: Any
    def __init__(self, auth_provider: TrustedNetworksAuthProvider, ip_addr: IPAddress, available_users: dict[str, Union[str, None]], allow_bypass_login: bool) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...



from forecast_solar.models import Estimate as Estimate
from homeassistant.components.sensor import SensorEntityDescription as SensorEntityDescription
from typing import Any, Callable

class ForecastSolarSensorEntityDescription(SensorEntityDescription):
    state: Union[Callable[[Estimate], Any], None]



from .const import DOMAIN as DOMAIN, KNX_ADDRESS as KNX_ADDRESS
from homeassistant.components.notify import BaseNotificationService as BaseNotificationService
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Notification as XknxNotification

async def async_get_service(hass: HomeAssistant, config: ConfigType, discovery_info: Union[DiscoveryInfoType, None] = ...) -> Union[KNXNotificationService, None]: ...

class KNXNotificationService(BaseNotificationService):
    devices: Any
    def __init__(self, devices: list[XknxNotification]) -> None: ...
    @property
    def targets(self) -> dict[str, str]: ...
    async def async_send_message(self, message: str = ..., **kwargs: Any) -> None: ...
    async def _async_send_to_all_devices(self, message: str) -> None: ...
    async def _async_send_to_device(self, message: str, names: str) -> None: ...



import types
from ..auth_store import AuthStore as AuthStore
from ..const import MFA_SESSION_EXPIRATION as MFA_SESSION_EXPIRATION
from ..models import Credentials as Credentials, RefreshToken as RefreshToken, User as User, UserMeta as UserMeta
from collections.abc import Mapping
from homeassistant import data_entry_flow as data_entry_flow, requirements as requirements
from homeassistant.const import CONF_ID as CONF_ID, CONF_NAME as CONF_NAME, CONF_TYPE as CONF_TYPE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.util.decorator import Registry as Registry
from typing import Any

_LOGGER: Any
DATA_REQS: str
AUTH_PROVIDERS: Any
AUTH_PROVIDER_SCHEMA: Any

class AuthProvider:
    DEFAULT_TITLE: str
    hass: Any
    store: Any
    config: Any
    def __init__(self, hass: HomeAssistant, store: AuthStore, config: dict[str, Any]) -> None: ...
    @property
    def id(self) -> Union[str, None]: ...
    @property
    def type(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def support_mfa(self) -> bool: ...
    async def async_credentials(self) -> list[Credentials]: ...
    def async_create_credentials(self, data: dict[str, str]) -> Credentials: ...
    async def async_login_flow(self, context: Union[dict, None]) -> LoginFlow: ...
    async def async_get_or_create_credentials(self, flow_result: Mapping[str, str]) -> Credentials: ...
    async def async_user_meta_for_credentials(self, credentials: Credentials) -> UserMeta: ...
    async def async_initialize(self) -> None: ...
    def async_validate_refresh_token(self, refresh_token: RefreshToken, remote_ip: Union[str, None] = ...) -> None: ...

async def auth_provider_from_config(hass: HomeAssistant, store: AuthStore, config: dict[str, Any]) -> AuthProvider: ...
async def load_auth_provider_module(hass: HomeAssistant, provider: str) -> types.ModuleType: ...

class LoginFlow(data_entry_flow.FlowHandler):
    _auth_provider: Any
    _auth_module_id: Any
    _auth_manager: Any
    available_mfa_modules: Any
    created_at: Any
    invalid_mfa_times: int
    user: Any
    credential: Any
    def __init__(self, auth_provider: AuthProvider) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_step_select_mfa_module(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_step_mfa(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    async def async_finish(self, flow_result: Any) -> FlowResult: ...



from .models import DSMRSensorEntityDescription as DSMRSensorEntityDescription
from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT
from homeassistant.const import DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, DEVICE_CLASS_VOLTAGE as DEVICE_CLASS_VOLTAGE
from homeassistant.util import dt as dt
from typing import Any

DOMAIN: str
LOGGER: Any
PLATFORMS: Any
CONF_DSMR_VERSION: str
CONF_RECONNECT_INTERVAL: str
CONF_PRECISION: str
CONF_TIME_BETWEEN_UPDATE: str
CONF_SERIAL_ID: str
CONF_SERIAL_ID_GAS: str
DEFAULT_DSMR_VERSION: str
DEFAULT_PORT: str
DEFAULT_PRECISION: int
DEFAULT_RECONNECT_INTERVAL: int
DEFAULT_TIME_BETWEEN_UPDATE: int
DATA_TASK: str
DEVICE_NAME_ENERGY: str
DEVICE_NAME_GAS: str
SENSORS: tuple[DSMRSensorEntityDescription, ...]



from . import DATA_COORDINATOR as DATA_COORDINATOR, DATA_TILE as DATA_TILE, DOMAIN as DOMAIN
from collections.abc import Awaitable
from homeassistant.components.device_tracker.config_entry import TrackerEntity as TrackerEntity
from homeassistant.components.device_tracker.const import SOURCE_TYPE_GPS as SOURCE_TYPE_GPS
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import ATTR_ATTRIBUTION as ATTR_ATTRIBUTION, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator
from pytile.tile import Tile as Tile
from typing import Any, Callable

_LOGGER: Any
ATTR_ALTITUDE: str
ATTR_CONNECTION_STATE: str
ATTR_IS_DEAD: str
ATTR_IS_LOST: str
ATTR_LAST_LOST_TIMESTAMP: str
ATTR_RING_STATE: str
ATTR_TILE_NAME: str
ATTR_VOIP_STATE: str
DEFAULT_ATTRIBUTION: str
DEFAULT_ICON: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...
async def async_setup_scanner(hass: HomeAssistant, config: ConfigType, async_see: Callable[..., Awaitable[None]], discovery_info: Union[dict[str, Any], None] = ...) -> bool: ...

class TileDeviceTracker(CoordinatorEntity, TrackerEntity):
    _attr_icon: Any
    _attr_extra_state_attributes: Any
    _attr_name: Any
    _attr_unique_id: Any
    _entry: Any
    _tile: Any
    def __init__(self, entry: ConfigEntry, coordinator: DataUpdateCoordinator, tile: Tile) -> None: ...
    @property
    def available(self) -> bool: ...
    @property
    def location_accuracy(self) -> int: ...
    @property
    def latitude(self) -> Union[float, None]: ...
    @property
    def longitude(self) -> Union[float, None]: ...
    @property
    def source_type(self) -> str: ...
    def _handle_coordinator_update(self) -> None: ...
    def _update_from_latest_data(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...



from .const import ADMIN_SERVICES as ADMIN_SERVICES, ALL_KEYS as ALL_KEYS, ATTR_UNIQUE_ID as ATTR_UNIQUE_ID, CONF_UNAUTHENTICATED_MODE as CONF_UNAUTHENTICATED_MODE, CONNECTION_TIMEOUT as CONNECTION_TIMEOUT, DEFAULT_DEVICE_NAME as DEFAULT_DEVICE_NAME, DEFAULT_NOTIFY_SERVICE_NAME as DEFAULT_NOTIFY_SERVICE_NAME, DOMAIN as DOMAIN, KEY_DEVICE_BASIC_INFORMATION as KEY_DEVICE_BASIC_INFORMATION, KEY_DEVICE_INFORMATION as KEY_DEVICE_INFORMATION, KEY_DEVICE_SIGNAL as KEY_DEVICE_SIGNAL, KEY_DIALUP_MOBILE_DATASWITCH as KEY_DIALUP_MOBILE_DATASWITCH, KEY_LAN_HOST_INFO as KEY_LAN_HOST_INFO, KEY_MONITORING_CHECK_NOTIFICATIONS as KEY_MONITORING_CHECK_NOTIFICATIONS, KEY_MONITORING_MONTH_STATISTICS as KEY_MONITORING_MONTH_STATISTICS, KEY_MONITORING_STATUS as KEY_MONITORING_STATUS, KEY_MONITORING_TRAFFIC_STATISTICS as KEY_MONITORING_TRAFFIC_STATISTICS, KEY_NET_CURRENT_PLMN as KEY_NET_CURRENT_PLMN, KEY_NET_NET_MODE as KEY_NET_NET_MODE, KEY_SMS_SMS_COUNT as KEY_SMS_SMS_COUNT, KEY_WLAN_HOST_LIST as KEY_WLAN_HOST_LIST, KEY_WLAN_WIFI_FEATURE_SWITCH as KEY_WLAN_WIFI_FEATURE_SWITCH, NOTIFY_SUPPRESS_TIMEOUT as NOTIFY_SUPPRESS_TIMEOUT, SERVICE_CLEAR_TRAFFIC_STATISTICS as SERVICE_CLEAR_TRAFFIC_STATISTICS, SERVICE_REBOOT as SERVICE_REBOOT, SERVICE_RESUME_INTEGRATION as SERVICE_RESUME_INTEGRATION, SERVICE_SUSPEND_INTEGRATION as SERVICE_SUSPEND_INTEGRATION, UPDATE_SIGNAL as UPDATE_SIGNAL
from .utils import get_device_macs as get_device_macs
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_MAC as CONF_MAC, CONF_NAME as CONF_NAME, CONF_PASSWORD as CONF_PASSWORD, CONF_RECIPIENT as CONF_RECIPIENT, CONF_URL as CONF_URL, CONF_USERNAME as CONF_USERNAME, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import discovery as discovery, entity_registry as entity_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, dispatcher_send as dispatcher_send
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo, Entity as Entity
from homeassistant.helpers.event import async_track_time_interval as async_track_time_interval
from homeassistant.helpers.typing import ConfigType as ConfigType
from huawei_lte_api.Client import Client
from huawei_lte_api.Connection import Connection
from typing import Any, Callable

_LOGGER: Any
SCAN_INTERVAL: Any
NOTIFY_SCHEMA: Any
CONFIG_SCHEMA: Any
SERVICE_SCHEMA: Any
CONFIG_ENTRY_PLATFORMS: Any

class Router:
    hass: HomeAssistant
    config_entry: ConfigEntry
    connection: Connection
    url: str
    data: dict[str, Any]
    subscriptions: dict[str, set[str]]
    inflight_gets: set[str]
    client: Client
    suspended: Any
    notify_last_attempt: float
    def __attrs_post_init__(self) -> None: ...
    @property
    def device_name(self) -> str: ...
    @property
    def device_identifiers(self) -> set[tuple[str, str]]: ...
    @property
    def device_connections(self) -> set[tuple[str, str]]: ...
    def _get_data(self, key: str, func: Callable[[], Any]) -> None: ...
    def update(self) -> None: ...
    def logout(self) -> None: ...
    def cleanup(self, *_: Any) -> None: ...
    def __init__(self, hass, config_entry, connection, url, data, subscriptions, inflight_gets, suspended, notify_last_attempt) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class HuaweiLteData:
    hass_config: dict
    config: dict[str, dict[str, Any]]
    routers: dict[str, Router]
    def __init__(self, hass_config, config, routers) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...
async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_migrate_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...

class HuaweiLteBaseEntity(Entity):
    router: Router
    _available: bool
    _unsub_handlers: list[Callable]
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def available(self) -> bool: ...
    @property
    def should_poll(self) -> bool: ...
    @property
    def device_info(self) -> DeviceInfo: ...
    async def async_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def _async_maybe_update(self, url: str) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    def __init__(self, router, available, unsub_handlers) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...



import pyatmo
from .const import ATTR_HEATING_POWER_REQUEST as ATTR_HEATING_POWER_REQUEST, ATTR_SCHEDULE_NAME as ATTR_SCHEDULE_NAME, ATTR_SELECTED_SCHEDULE as ATTR_SELECTED_SCHEDULE, DATA_DEVICE_IDS as DATA_DEVICE_IDS, DATA_HANDLER as DATA_HANDLER, DATA_HOMES as DATA_HOMES, DATA_SCHEDULES as DATA_SCHEDULES, DOMAIN as DOMAIN, EVENT_TYPE_CANCEL_SET_POINT as EVENT_TYPE_CANCEL_SET_POINT, EVENT_TYPE_SCHEDULE as EVENT_TYPE_SCHEDULE, EVENT_TYPE_SET_POINT as EVENT_TYPE_SET_POINT, EVENT_TYPE_THERM_MODE as EVENT_TYPE_THERM_MODE, MANUFACTURER as MANUFACTURER, SERVICE_SET_SCHEDULE as SERVICE_SET_SCHEDULE, SIGNAL_NAME as SIGNAL_NAME
from .data_handler import HOMEDATA_DATA_CLASS_NAME as HOMEDATA_DATA_CLASS_NAME, HOMESTATUS_DATA_CLASS_NAME as HOMESTATUS_DATA_CLASS_NAME, NetatmoDataHandler as NetatmoDataHandler
from .netatmo_entity_base import NetatmoBase as NetatmoBase
from homeassistant.components.climate import ClimateEntity as ClimateEntity
from homeassistant.components.climate.const import CURRENT_HVAC_HEAT as CURRENT_HVAC_HEAT, CURRENT_HVAC_IDLE as CURRENT_HVAC_IDLE, DEFAULT_MIN_TEMP as DEFAULT_MIN_TEMP, HVAC_MODE_AUTO as HVAC_MODE_AUTO, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, PRESET_BOOST as PRESET_BOOST, SUPPORT_PRESET_MODE as SUPPORT_PRESET_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_BATTERY_LEVEL as ATTR_BATTERY_LEVEL, ATTR_TEMPERATURE as ATTR_TEMPERATURE, PRECISION_HALVES as PRECISION_HALVES, STATE_OFF as STATE_OFF, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import PlatformNotReady as PlatformNotReady
from homeassistant.helpers import entity_platform as entity_platform
from homeassistant.helpers.device_registry import async_get_registry as async_get_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any
PRESET_FROST_GUARD: str
PRESET_SCHEDULE: str
PRESET_MANUAL: str
SUPPORT_FLAGS: Any
SUPPORT_HVAC: Any
SUPPORT_PRESET: Any
STATE_NETATMO_SCHEDULE: str
STATE_NETATMO_HG: str
STATE_NETATMO_MAX: str
STATE_NETATMO_AWAY = PRESET_AWAY
STATE_NETATMO_OFF = STATE_OFF
STATE_NETATMO_MANUAL: str
STATE_NETATMO_HOME: str
PRESET_MAP_NETATMO: Any
NETATMO_MAP_PRESET: Any
HVAC_MAP_NETATMO: Any
CURRENT_HVAC_MAP_NETATMO: Any
DEFAULT_MAX_TEMP: int
NA_THERM: str
NA_VALVE: str
SUGGESTED_AREA: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class NetatmoThermostat(NetatmoBase, ClimateEntity):
    _id: Any
    _home_id: Any
    _home_status_class: Any
    _home_status: Any
    _room_status: Any
    _room_data: Any
    _model: Any
    _device_name: Any
    _attr_name: Any
    _current_temperature: Any
    _target_temperature: Any
    _preset: Any
    _away: Any
    _operation_list: Any
    _support_flags: Any
    _hvac_mode: Any
    _battery_level: Any
    _connected: Any
    _away_temperature: Any
    _hg_temperature: Any
    _boilerstatus: Any
    _setpoint_duration: Any
    _selected_schedule: Any
    _attr_unique_id: Any
    def __init__(self, data_handler: NetatmoDataHandler, home_id: str, room_id: str) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def handle_event(self, event: dict) -> None: ...
    @property
    def _data(self) -> pyatmo.AsyncHomeData: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature_step(self) -> Union[float, None]: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    @property
    def hvac_action(self) -> Union[str, None]: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    async def async_set_preset_mode(self, preset_mode: str) -> None: ...
    @property
    def preset_mode(self) -> Union[str, None]: ...
    @property
    def preset_modes(self) -> Union[list[str], None]: ...
    async def async_set_temperature(self, **kwargs: dict) -> None: ...
    async def async_turn_off(self) -> None: ...
    async def async_turn_on(self) -> None: ...
    @property
    def available(self) -> bool: ...
    def async_update_callback(self) -> None: ...
    def _build_room_status(self) -> dict: ...
    async def _async_service_set_schedule(self, **kwargs: dict) -> None: ...
    @property
    def device_info(self) -> DeviceInfo: ...

def get_all_home_ids(home_data: Union[pyatmo.HomeData, None]) -> list[str]: ...



from . import ATTR_BRIGHTNESS as ATTR_BRIGHTNESS, ATTR_COLOR_TEMP as ATTR_COLOR_TEMP, ATTR_EFFECT as ATTR_EFFECT, ATTR_HS_COLOR as ATTR_HS_COLOR, ATTR_WHITE_VALUE as ATTR_WHITE_VALUE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.significant_change import check_numeric_changed as check_numeric_changed, either_one_none as either_one_none
from typing import Any

def async_check_significant_change(hass: HomeAssistant, old_state: str, old_attrs: dict, new_state: str, new_attrs: dict, **kwargs: Any) -> Union[bool, None]: ...



from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_CODE as ATTR_CODE, ATTR_CODE_FORMAT as ATTR_CODE_FORMAT, SERVICE_LOCK as SERVICE_LOCK, SERVICE_OPEN as SERVICE_OPEN, SERVICE_UNLOCK as SERVICE_UNLOCK, STATE_JAMMED as STATE_JAMMED, STATE_LOCKED as STATE_LOCKED, STATE_LOCKING as STATE_LOCKING, STATE_UNLOCKED as STATE_UNLOCKED, STATE_UNLOCKING as STATE_UNLOCKING
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE, make_entity_service_schema as make_entity_service_schema
from homeassistant.helpers.entity import Entity as Entity, EntityDescription as EntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.typing import ConfigType as ConfigType, StateType as StateType
from typing import Any

_LOGGER: Any
ATTR_CHANGED_BY: str
DOMAIN: str
SCAN_INTERVAL: Any
ENTITY_ID_FORMAT: Any
MIN_TIME_BETWEEN_SCANS: Any
LOCK_SERVICE_SCHEMA: Any
SUPPORT_OPEN: int
PROP_TO_ATTR: Any

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class LockEntityDescription(EntityDescription): ...

class LockEntity(Entity):
    entity_description: LockEntityDescription
    _attr_changed_by: Union[str, None]
    _attr_code_format: Union[str, None]
    _attr_is_locked: Union[bool, None]
    _attr_is_locking: Union[bool, None]
    _attr_is_unlocking: Union[bool, None]
    _attr_is_jammed: Union[bool, None]
    _attr_state: None
    @property
    def changed_by(self) -> Union[str, None]: ...
    @property
    def code_format(self) -> Union[str, None]: ...
    @property
    def is_locked(self) -> Union[bool, None]: ...
    @property
    def is_locking(self) -> Union[bool, None]: ...
    @property
    def is_unlocking(self) -> Union[bool, None]: ...
    @property
    def is_jammed(self) -> Union[bool, None]: ...
    def lock(self, **kwargs: Any) -> None: ...
    async def async_lock(self, **kwargs: Any) -> None: ...
    def unlock(self, **kwargs: Any) -> None: ...
    async def async_unlock(self, **kwargs: Any) -> None: ...
    def open(self, **kwargs: Any) -> None: ...
    async def async_open(self, **kwargs: Any) -> None: ...
    @property
    def state_attributes(self) -> dict[str, StateType]: ...
    @property
    def state(self) -> Union[str, None]: ...

class LockDevice(LockEntity):
    def __init_subclass__(cls, **kwargs: Any): ...



from .typing import UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from collections.abc import Iterable, Mapping
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_DEVICE_CLASS as ATTR_DEVICE_CLASS, ATTR_FRIENDLY_NAME as ATTR_FRIENDLY_NAME, ATTR_ICON as ATTR_ICON, ATTR_RESTORED as ATTR_RESTORED, ATTR_SUPPORTED_FEATURES as ATTR_SUPPORTED_FEATURES, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START, MAX_LENGTH_STATE_DOMAIN as MAX_LENGTH_STATE_DOMAIN, MAX_LENGTH_STATE_ENTITY_ID as MAX_LENGTH_STATE_ENTITY_ID, STATE_UNAVAILABLE as STATE_UNAVAILABLE
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback, split_entity_id as split_entity_id, valid_entity_id as valid_entity_id
from homeassistant.exceptions import MaxLengthExceeded as MaxLengthExceeded
from homeassistant.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED as EVENT_DEVICE_REGISTRY_UPDATED
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util import slugify as slugify
from homeassistant.util.yaml import load_yaml as load_yaml
from typing import Any, Callable

PATH_REGISTRY: str
DATA_REGISTRY: str
EVENT_ENTITY_REGISTRY_UPDATED: str
SAVE_DELAY: int
_LOGGER: Any
DISABLED_CONFIG_ENTRY: str
DISABLED_DEVICE: str
DISABLED_HASS: str
DISABLED_INTEGRATION: str
DISABLED_USER: str
STORAGE_VERSION: int
STORAGE_KEY: str
ENTITY_DESCRIBING_ATTRIBUTES: Any

class RegistryEntry:
    entity_id: str
    unique_id: str
    platform: str
    name: Union[str, None]
    icon: Union[str, None]
    device_id: Union[str, None]
    area_id: Union[str, None]
    config_entry_id: Union[str, None]
    disabled_by: Union[str, None]
    capabilities: Union[Mapping[str, Any], None]
    supported_features: int
    device_class: Union[str, None]
    unit_of_measurement: Union[str, None]
    original_name: Union[str, None]
    original_icon: Union[str, None]
    domain: str
    def _domain_default(self) -> str: ...
    @property
    def disabled(self) -> bool: ...
    def write_unavailable_state(self, hass: HomeAssistant) -> None: ...
    def __init__(self, entity_id, unique_id, platform, name, icon, device_id, area_id, config_entry_id, disabled_by, capabilities, supported_features, device_class, unit_of_measurement, original_name, original_icon, domain) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class EntityRegistry:
    hass: Any
    entities: Any
    _index: Any
    _store: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_get_device_class_lookup(self, domain_device_classes: set[tuple[str, Union[str, None]]]) -> dict: ...
    def async_is_registered(self, entity_id: str) -> bool: ...
    def async_get(self, entity_id: str) -> Union[RegistryEntry, None]: ...
    def async_get_entity_id(self, domain: str, platform: str, unique_id: str) -> Union[str, None]: ...
    def async_generate_entity_id(self, domain: str, suggested_object_id: str, known_object_ids: Union[Iterable[str], None] = ...) -> str: ...
    def async_get_or_create(self, domain: str, platform: str, unique_id: str, *, suggested_object_id: Union[str, None] = ..., known_object_ids: Union[Iterable[str], None] = ..., disabled_by: Union[str, None] = ..., config_entry: Union[ConfigEntry, None] = ..., device_id: Union[str, None] = ..., area_id: Union[str, None] = ..., capabilities: Union[Mapping[str, Any], None] = ..., supported_features: Union[int, None] = ..., device_class: Union[str, None] = ..., unit_of_measurement: Union[str, None] = ..., original_name: Union[str, None] = ..., original_icon: Union[str, None] = ...) -> RegistryEntry: ...
    def async_remove(self, entity_id: str) -> None: ...
    def async_device_modified(self, event: Event) -> None: ...
    def async_update_entity(self, entity_id: str, *, name: Union[str, None, UndefinedType] = ..., icon: Union[str, None, UndefinedType] = ..., area_id: Union[str, None, UndefinedType] = ..., new_entity_id: Union[str, UndefinedType] = ..., new_unique_id: Union[str, UndefinedType] = ..., disabled_by: Union[str, None, UndefinedType] = ...) -> RegistryEntry: ...
    def _async_update_entity(self, entity_id: str, *, name: Union[str, None, UndefinedType] = ..., icon: Union[str, None, UndefinedType] = ..., config_entry_id: Union[str, None, UndefinedType] = ..., new_entity_id: Union[str, UndefinedType] = ..., device_id: Union[str, None, UndefinedType] = ..., area_id: Union[str, None, UndefinedType] = ..., new_unique_id: Union[str, UndefinedType] = ..., disabled_by: Union[str, None, UndefinedType] = ..., capabilities: Union[Mapping[str, Any], None, UndefinedType] = ..., supported_features: Union[int, UndefinedType] = ..., device_class: Union[str, None, UndefinedType] = ..., unit_of_measurement: Union[str, None, UndefinedType] = ..., original_name: Union[str, None, UndefinedType] = ..., original_icon: Union[str, None, UndefinedType] = ...) -> RegistryEntry: ...
    async def async_load(self) -> None: ...
    def async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict[str, Any]: ...
    def async_clear_config_entry(self, config_entry: str) -> None: ...
    def async_clear_area_id(self, area_id: str) -> None: ...
    def _register_entry(self, entry: RegistryEntry) -> None: ...
    def _add_index(self, entry: RegistryEntry) -> None: ...
    def _unregister_entry(self, entry: RegistryEntry) -> None: ...
    def _remove_index(self, entry: RegistryEntry) -> None: ...
    def _rebuild_index(self) -> None: ...

def async_get(hass: HomeAssistant) -> EntityRegistry: ...
async def async_load(hass: HomeAssistant) -> None: ...
async def async_get_registry(hass: HomeAssistant) -> EntityRegistry: ...
def async_entries_for_device(registry: EntityRegistry, device_id: str, include_disabled_entities: bool = ...) -> list[RegistryEntry]: ...
def async_entries_for_area(registry: EntityRegistry, area_id: str) -> list[RegistryEntry]: ...
def async_entries_for_config_entry(registry: EntityRegistry, config_entry_id: str) -> list[RegistryEntry]: ...
def async_config_entry_disabled_by_changed(registry: EntityRegistry, config_entry: ConfigEntry) -> None: ...
async def _async_migrate(entities: dict[str, Any]) -> dict[str, list[dict[str, Any]]]: ...
def async_setup_entity_restore(hass: HomeAssistant, registry: EntityRegistry) -> None: ...
async def async_migrate_entries(hass: HomeAssistant, config_entry_id: str, entry_callback: Callable[[RegistryEntry], Union[dict, None]]) -> None: ...



from collections.abc import Awaitable
from homeassistant.const import EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from typing import Callable

def async_at_start(hass: HomeAssistant, at_start_cb: Callable[[HomeAssistant], Awaitable]) -> None: ...



from .const import DATA_SUBSCRIBER as DATA_SUBSCRIBER, DOMAIN as DOMAIN
from .events import DEVICE_TRAIT_TRIGGER_MAP as DEVICE_TRAIT_TRIGGER_MAP, NEST_EVENT as NEST_EVENT
from homeassistant.components.automation import AutomationActionType as AutomationActionType
from homeassistant.components.device_automation import DEVICE_TRIGGER_BASE_SCHEMA as DEVICE_TRIGGER_BASE_SCHEMA
from homeassistant.components.device_automation.exceptions import InvalidDeviceAutomationConfig as InvalidDeviceAutomationConfig
from homeassistant.const import CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_DOMAIN as CONF_DOMAIN, CONF_PLATFORM as CONF_PLATFORM, CONF_TYPE as CONF_TYPE
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant
from homeassistant.helpers.device_registry import DeviceRegistry as DeviceRegistry
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

DEVICE: str
TRIGGER_TYPES: Any
TRIGGER_SCHEMA: Any

async def async_get_nest_device_id(hass: HomeAssistant, device_id: str) -> Union[str, None]: ...
async def async_get_device_trigger_types(hass: HomeAssistant, nest_device_id: str) -> list[str]: ...
async def async_get_triggers(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_attach_trigger(hass: HomeAssistant, config: ConfigType, action: AutomationActionType, automation_info: dict) -> CALLBACK_TYPE: ...



import re
from . import HuaweiLteBaseEntity as HuaweiLteBaseEntity
from .const import DOMAIN as DOMAIN, KEY_DEVICE_INFORMATION as KEY_DEVICE_INFORMATION, KEY_DEVICE_SIGNAL as KEY_DEVICE_SIGNAL, KEY_MONITORING_CHECK_NOTIFICATIONS as KEY_MONITORING_CHECK_NOTIFICATIONS, KEY_MONITORING_MONTH_STATISTICS as KEY_MONITORING_MONTH_STATISTICS, KEY_MONITORING_STATUS as KEY_MONITORING_STATUS, KEY_MONITORING_TRAFFIC_STATISTICS as KEY_MONITORING_TRAFFIC_STATISTICS, KEY_NET_CURRENT_PLMN as KEY_NET_CURRENT_PLMN, KEY_NET_NET_MODE as KEY_NET_NET_MODE, KEY_SMS_SMS_COUNT as KEY_SMS_SMS_COUNT, SENSOR_KEYS as SENSOR_KEYS
from homeassistant.components.sensor import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_SIGNAL_STRENGTH as DEVICE_CLASS_SIGNAL_STRENGTH, SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DATA_BYTES as DATA_BYTES, DATA_RATE_BYTES_PER_SECOND as DATA_RATE_BYTES_PER_SECOND, FREQUENCY_MEGAHERTZ as FREQUENCY_MEGAHERTZ, PERCENTAGE as PERCENTAGE, STATE_UNKNOWN as STATE_UNKNOWN, TIME_SECONDS as TIME_SECONDS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from typing import Any, Callable, NamedTuple

_LOGGER: Any

class SensorMeta(NamedTuple):
    name: Union[str, None]
    device_class: Union[str, None]
    icon: Union[str, Callable[[StateType], str], None]
    unit: Union[str, None]
    enabled_default: bool
    include: Union[re.Pattern[str], None]
    exclude: Union[re.Pattern[str], None]
    formatter: Union[Callable[[str], tuple[StateType, Union[str, None]]], None]

SENSOR_META: dict[Union[str, tuple[str, str]], SensorMeta]

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...
def format_default(value: StateType) -> tuple[StateType, Union[str, None]]: ...

class HuaweiLteSensor(HuaweiLteBaseEntity, SensorEntity):
    key: str
    item: str
    meta: SensorMeta
    _state: StateType
    _unit: Union[str, None]
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    @property
    def _entity_name(self) -> str: ...
    @property
    def _device_unique_id(self) -> str: ...
    @property
    def state(self) -> StateType: ...
    @property
    def device_class(self) -> Union[str, None]: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...
    @property
    def icon(self) -> Union[str, None]: ...
    @property
    def entity_registry_enabled_default(self) -> bool: ...
    _available: Any
    async def async_update(self) -> None: ...
    def __init__(self, router, available, unsub_handlers, key, item, meta, state, unit) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...



from .const import DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .entity import ZWaveBaseEntity as ZWaveBaseEntity
from homeassistant.components.lock import LockEntity as LockEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_LOCKED as STATE_LOCKED, STATE_UNLOCKED as STATE_UNLOCKED
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import entity_platform as entity_platform
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

LOGGER: Any
STATE_TO_ZWAVE_MAP: dict[int, dict[str, Union[int, bool]]]
SERVICE_SET_LOCK_USERCODE: str
SERVICE_CLEAR_LOCK_USERCODE: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ZWaveLock(ZWaveBaseEntity, LockEntity):
    @property
    def is_locked(self) -> Union[bool, None]: ...
    async def _set_lock_state(self, target_state: str, **kwargs: dict[str, Any]) -> None: ...
    async def async_lock(self, **kwargs: dict[str, Any]) -> None: ...
    async def async_unlock(self, **kwargs: dict[str, Any]) -> None: ...
    async def async_set_lock_usercode(self, code_slot: int, usercode: str) -> None: ...
    async def async_clear_lock_usercode(self, code_slot: int) -> None: ...



from .const import DOMAIN as DOMAIN
from .discovery import ZwaveDiscoveryInfo as ZwaveDiscoveryInfo
from .helpers import get_device_id as get_device_id, get_unique_id as get_unique_id
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import Entity as Entity
from typing import Any
from zwave_js_server.client import Client as ZwaveClient
from zwave_js_server.model.value import Value as ZwaveValue

LOGGER: Any
EVENT_VALUE_UPDATED: str
EVENT_DEAD: str
EVENT_ALIVE: str

class ZWaveBaseEntity(Entity):
    config_entry: Any
    client: Any
    info: Any
    watched_value_ids: Any
    _attr_name: Any
    _attr_unique_id: Any
    _attr_entity_registry_enabled_default: Any
    _attr_assumed_state: Any
    _attr_device_info: Any
    def __init__(self, config_entry: ConfigEntry, client: ZwaveClient, info: ZwaveDiscoveryInfo) -> None: ...
    def on_value_update(self) -> None: ...
    async def async_poll_value(self, refresh_all_values: bool) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    def generate_name(self, include_value_name: bool = ..., alternate_value_name: Union[str, None] = ..., additional_info: Union[list[str], None] = ..., name_suffix: Union[str, None] = ...) -> str: ...
    @property
    def available(self) -> bool: ...
    def _node_status_alive_or_dead(self, event_data: dict) -> None: ...
    def _value_changed(self, event_data: dict) -> None: ...
    def get_zwave_value(self, value_property: Union[str, int], command_class: Union[int, None] = ..., endpoint: Union[int, None] = ..., value_property_key: Union[int, None] = ..., add_to_watched_value_ids: bool = ..., check_all_endpoints: bool = ...) -> Union[ZwaveValue, None]: ...
    @property
    def should_poll(self) -> bool: ...



from . import DOMAIN as DOMAIN
from homeassistant.components.automation import AutomationActionType as AutomationActionType
from homeassistant.components.device_automation import toggle_entity as toggle_entity
from homeassistant.const import CONF_DOMAIN as CONF_DOMAIN
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

TRIGGER_SCHEMA: Any

async def async_attach_trigger(hass: HomeAssistant, config: ConfigType, action: AutomationActionType, automation_info: dict) -> CALLBACK_TYPE: ...
async def async_get_triggers(hass: HomeAssistant, device_id: str) -> list[dict]: ...
async def async_get_trigger_capabilities(hass: HomeAssistant, config: dict) -> dict: ...



from .const import CONF_AZIMUTH as CONF_AZIMUTH, CONF_DAMPING as CONF_DAMPING, CONF_DECLINATION as CONF_DECLINATION, CONF_MODULES_POWER as CONF_MODULES_POWER, DOMAIN as DOMAIN
from homeassistant.components import websocket_api as websocket_api
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_API_KEY as CONF_API_KEY, CONF_LATITUDE as CONF_LATITUDE, CONF_LONGITUDE as CONF_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

PLATFORMS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_update_options(hass: HomeAssistant, entry: ConfigEntry) -> None: ...
def ws_list_forecasts(hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...



from . import SwitcherDeviceWrapper as SwitcherDeviceWrapper
from .const import CONF_AUTO_OFF as CONF_AUTO_OFF, CONF_TIMER_MINUTES as CONF_TIMER_MINUTES, SERVICE_SET_AUTO_OFF_NAME as SERVICE_SET_AUTO_OFF_NAME, SERVICE_TURN_ON_WITH_TIMER_NAME as SERVICE_TURN_ON_WITH_TIMER_NAME, SIGNAL_DEVICE_ADD as SIGNAL_DEVICE_ADD
from aioswitcher.api import SwitcherBaseResponse as SwitcherBaseResponse
from datetime import timedelta
from homeassistant.components.switch import DEVICE_CLASS_OUTLET as DEVICE_CLASS_OUTLET, DEVICE_CLASS_SWITCH as DEVICE_CLASS_SWITCH, SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import device_registry as device_registry, entity_platform as entity_platform
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

_LOGGER: Any
SERVICE_SET_AUTO_OFF_SCHEMA: Any
SERVICE_TURN_ON_WITH_TIMER_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class SwitcherBaseSwitchEntity(CoordinatorEntity, SwitchEntity):
    wrapper: Any
    control_result: Any
    _attr_name: Any
    _attr_unique_id: Any
    _attr_device_info: Any
    def __init__(self, wrapper: SwitcherDeviceWrapper) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    async def _async_call_api(self, api: str, *args: Any) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: dict) -> None: ...
    async def async_turn_off(self, **kwargs: dict) -> None: ...
    async def async_set_auto_off_service(self, auto_off: timedelta) -> None: ...
    async def async_turn_on_with_timer_service(self, timer_minutes: int) -> None: ...

class SwitcherPowerPlugSwitchEntity(SwitcherBaseSwitchEntity):
    _attr_device_class: Any

class SwitcherWaterHeaterSwitchEntity(SwitcherBaseSwitchEntity):
    _attr_device_class: Any
    async def async_set_auto_off_service(self, auto_off: timedelta) -> None: ...
    control_result: bool
    async def async_turn_on_with_timer_service(self, timer_minutes: int) -> None: ...



from . import LcnEntity as LcnEntity
from .const import CONF_DOMAIN_DATA as CONF_DOMAIN_DATA, CONF_OUTPUT as CONF_OUTPUT, OUTPUT_PORTS as OUTPUT_PORTS
from .helpers import DeviceConnectionType as DeviceConnectionType, InputType as InputType, get_device_connection as get_device_connection
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_ADDRESS as CONF_ADDRESS, CONF_DOMAIN as CONF_DOMAIN, CONF_ENTITIES as CONF_ENTITIES
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

PARALLEL_UPDATES: int

def create_lcn_switch_entity(hass: HomeAssistant, entity_config: ConfigType, config_entry: ConfigEntry) -> LcnEntity: ...
async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class LcnOutputSwitch(LcnEntity, SwitchEntity):
    output: Any
    _is_on: bool
    def __init__(self, config: ConfigType, entry_id: str, device_connection: DeviceConnectionType) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    def input_received(self, input_obj: InputType) -> None: ...

class LcnRelaySwitch(LcnEntity, SwitchEntity):
    output: Any
    _is_on: bool
    def __init__(self, config: ConfigType, entry_id: str, device_connection: DeviceConnectionType) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    def input_received(self, input_obj: InputType) -> None: ...



from .model import GiosSensorEntityDescription as GiosSensorEntityDescription
from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT
from homeassistant.const import CONCENTRATION_MICROGRAMS_PER_CUBIC_METER as CONCENTRATION_MICROGRAMS_PER_CUBIC_METER
from typing import Any, Final

ATTRIBUTION: Final[str]
CONF_STATION_ID: Final[str]
DEFAULT_NAME: Final[str]
SCAN_INTERVAL: Final[Any]
DOMAIN: Final[str]
MANUFACTURER: Final[str]
API_TIMEOUT: Final[int]
ATTR_INDEX: Final[str]
ATTR_STATION: Final[str]
ATTR_C6H6: Final[str]
ATTR_CO: Final[str]
ATTR_NO2: Final[str]
ATTR_O3: Final[str]
ATTR_PM10: Final[str]
ATTR_PM25: Final[str]
ATTR_SO2: Final[str]
ATTR_AQI: Final[str]
SENSOR_TYPES: Final[tuple[GiosSensorEntityDescription, ...]]



from homeassistant.components.sensor import ATTR_LAST_RESET as ATTR_LAST_RESET, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, PLATFORM_SCHEMA as PLATFORM_SCHEMA, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntity as SensorEntity
from homeassistant.const import ATTR_DEVICE_CLASS as ATTR_DEVICE_CLASS, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, CONF_METHOD as CONF_METHOD, CONF_NAME as CONF_NAME, STATE_UNAVAILABLE as STATE_UNAVAILABLE, STATE_UNKNOWN as STATE_UNKNOWN, TIME_DAYS as TIME_DAYS, TIME_HOURS as TIME_HOURS, TIME_MINUTES as TIME_MINUTES, TIME_SECONDS as TIME_SECONDS
from homeassistant.core import callback as callback
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from typing import Any

_LOGGER: Any
ATTR_SOURCE_ID: str
CONF_SOURCE_SENSOR: str
CONF_ROUND_DIGITS: str
CONF_UNIT_PREFIX: str
CONF_UNIT_TIME: str
CONF_UNIT_OF_MEASUREMENT: str
TRAPEZOIDAL_METHOD: str
LEFT_METHOD: str
RIGHT_METHOD: str
INTEGRATION_METHOD: Any
UNIT_PREFIXES: Any
UNIT_TIME: Any
ICON: str
DEFAULT_ROUND: int

async def async_setup_platform(hass, config, async_add_entities, discovery_info: Any | None = ...) -> None: ...

class IntegrationSensor(RestoreEntity, SensorEntity):
    _sensor_source_id: Any
    _round_digits: Any
    _state: int
    _method: Any
    _name: Any
    _unit_template: Any
    _unit_of_measurement: Any
    _unit_prefix: Any
    _unit_time: Any
    _attr_state_class: Any
    def __init__(self, source_entity, name, round_digits, unit_prefix, unit_time, unit_of_measurement, integration_method) -> None: ...
    _attr_last_reset: Any
    _attr_device_class: Any
    async def async_added_to_hass(self) -> None: ...
    @property
    def name(self): ...
    @property
    def state(self): ...
    @property
    def unit_of_measurement(self): ...
    @property
    def should_poll(self): ...
    @property
    def extra_state_attributes(self): ...
    @property
    def icon(self): ...



from homeassistant.const import TEMPERATURE as TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, TEMP_KELVIN as TEMP_KELVIN, UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE

def fahrenheit_to_celsius(fahrenheit: float, interval: bool = ...) -> float: ...
def kelvin_to_celsius(kelvin: float, interval: bool = ...) -> float: ...
def celsius_to_fahrenheit(celsius: float, interval: bool = ...) -> float: ...
def celsius_to_kelvin(celsius: float, interval: bool = ...) -> float: ...
def convert(temperature: float, from_unit: str, to_unit: str, interval: bool = ...) -> float: ...



from .const import CONF_DATA_COLLECTION_OPTED_IN as CONF_DATA_COLLECTION_OPTED_IN, DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN, EVENT_DEVICE_ADDED_TO_REGISTRY as EVENT_DEVICE_ADDED_TO_REGISTRY
from .helpers import async_enable_statistics as async_enable_statistics, update_data_collection_preference as update_data_collection_preference
from .services import BITMASK_SCHEMA as BITMASK_SCHEMA
from aiohttp import web
from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.http.view import HomeAssistantView as HomeAssistantView
from homeassistant.components.websocket_api.connection import ActiveConnection as ActiveConnection
from homeassistant.components.websocket_api.const import ERR_NOT_FOUND as ERR_NOT_FOUND, ERR_NOT_SUPPORTED as ERR_NOT_SUPPORTED, ERR_UNKNOWN_ERROR as ERR_UNKNOWN_ERROR
from homeassistant.config_entries import ConfigEntry as ConfigEntry, ConfigEntryState as ConfigEntryState
from homeassistant.const import CONF_URL as CONF_URL
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import Unauthorized as Unauthorized
from homeassistant.helpers.aiohttp_client import async_get_clientsession as async_get_clientsession
from homeassistant.helpers.device_registry import DeviceEntry as DeviceEntry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from typing import Callable
from zwave_js_server.client import Client as Client
from zwave_js_server.model.controller import ControllerStatistics as ControllerStatistics
from zwave_js_server.model.firmware import FirmwareUpdateFinished as FirmwareUpdateFinished, FirmwareUpdateProgress as FirmwareUpdateProgress
from zwave_js_server.model.log_message import LogMessage as LogMessage
from zwave_js_server.model.node import Node as Node, NodeStatistics as NodeStatistics

DATA_UNSUBSCRIBE: str
ID: str
ENTRY_ID: str
ERR_NOT_LOADED: str
NODE_ID: str
COMMAND_CLASS_ID: str
TYPE: str
PROPERTY: str
PROPERTY_KEY: str
VALUE: str
SECURE: str
CONFIG: str
LEVEL: str
LOG_TO_FILE: str
FILENAME: str
ENABLED: str
FORCE_CONSOLE: str
VALUE_ID: str
STATUS: str
OPTED_IN: str

def async_get_entry(orig_func: Callable) -> Callable: ...
def async_get_node(orig_func: Callable) -> Callable: ...
def async_handle_failed_command(orig_func: Callable) -> Callable: ...
def async_register_api(hass: HomeAssistant) -> None: ...
async def websocket_network_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_node_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_node_state(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_node_metadata(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_ping_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_add_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_stop_inclusion(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_stop_exclusion(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_remove_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_replace_failed_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_remove_failed_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_begin_healing_network(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_subscribe_heal_network_progress(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_stop_healing_network(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_heal_node(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_refresh_node_info(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_refresh_node_values(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_refresh_node_cc_values(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_set_config_parameter(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
async def websocket_get_config_parameters(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
def filename_is_present_if_logging_to_file(obj: dict) -> dict: ...
async def websocket_subscribe_log_updates(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_update_log_config(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_get_log_config(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_update_data_collection_preference(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_data_collection_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...

class DumpView(HomeAssistantView):
    url: str
    name: str
    async def get(self, request: web.Request, config_entry_id: str) -> web.Response: ...

async def websocket_version_info(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_abort_firmware_update(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...
def _get_firmware_update_progress_dict(progress: FirmwareUpdateProgress) -> dict[str, int]: ...
async def websocket_subscribe_firmware_update_status(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...

class FirmwareUploadView(HomeAssistantView):
    url: str
    name: str
    async def post(self, request: web.Request, config_entry_id: str, node_id: str) -> web.Response: ...

async def websocket_check_for_config_updates(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
async def websocket_install_config_update(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
def _get_controller_statistics_dict(statistics: ControllerStatistics) -> dict[str, int]: ...
async def websocket_subscribe_controller_statistics(hass: HomeAssistant, connection: ActiveConnection, msg: dict, entry: ConfigEntry, client: Client) -> None: ...
def _get_node_statistics_dict(statistics: NodeStatistics) -> dict[str, int]: ...
async def websocket_subscribe_node_statistics(hass: HomeAssistant, connection: ActiveConnection, msg: dict, node: Node) -> None: ...



import asyncio
import enum
from .async_ import run_callback_threadsafe as run_callback_threadsafe
from types import TracebackType
from typing import Any

ZONE_GLOBAL: str

class _State(str, enum.Enum):
    INIT: str
    ACTIVE: str
    TIMEOUT: str
    EXIT: str

class _GlobalFreezeContext:
    _loop: Any
    _manager: Any
    def __init__(self, manager: TimeoutManager) -> None: ...
    async def __aenter__(self) -> _GlobalFreezeContext: ...
    async def __aexit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    def __enter__(self) -> _GlobalFreezeContext: ...
    def __exit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    def _enter(self) -> None: ...
    def _exit(self) -> None: ...

class _ZoneFreezeContext:
    _loop: Any
    _zone: Any
    def __init__(self, zone: _ZoneTimeoutManager) -> None: ...
    async def __aenter__(self) -> _ZoneFreezeContext: ...
    async def __aexit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    def __enter__(self) -> _ZoneFreezeContext: ...
    def __exit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    def _enter(self) -> None: ...
    def _exit(self) -> None: ...

class _GlobalTaskContext:
    _loop: Any
    _manager: Any
    _task: Any
    _time_left: Any
    _expiration_time: Any
    _timeout_handler: Any
    _wait_zone: Any
    _state: Any
    _cool_down: Any
    def __init__(self, manager: TimeoutManager, task: asyncio.Task[Any], timeout: float, cool_down: float) -> None: ...
    async def __aenter__(self) -> _GlobalTaskContext: ...
    async def __aexit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    @property
    def state(self) -> _State: ...
    def zones_done_signal(self) -> None: ...
    def _start_timer(self) -> None: ...
    def _stop_timer(self) -> None: ...
    def _on_timeout(self) -> None: ...
    def _cancel_task(self) -> None: ...
    def pause(self) -> None: ...
    def reset(self) -> None: ...
    async def _on_wait(self) -> None: ...

class _ZoneTaskContext:
    _loop: Any
    _zone: Any
    _task: Any
    _state: Any
    _time_left: Any
    _expiration_time: Any
    _timeout_handler: Any
    def __init__(self, zone: _ZoneTimeoutManager, task: asyncio.Task[Any], timeout: float) -> None: ...
    @property
    def state(self) -> _State: ...
    async def __aenter__(self) -> _ZoneTaskContext: ...
    async def __aexit__(self, exc_type: type[BaseException], exc_val: BaseException, exc_tb: TracebackType) -> Union[bool, None]: ...
    def _start_timer(self) -> None: ...
    def _stop_timer(self) -> None: ...
    def _on_timeout(self) -> None: ...
    def pause(self) -> None: ...
    def reset(self) -> None: ...

class _ZoneTimeoutManager:
    _manager: Any
    _zone: Any
    _tasks: Any
    _freezes: Any
    def __init__(self, manager: TimeoutManager, zone: str) -> None: ...
    def __repr__(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def active(self) -> bool: ...
    @property
    def freezes_done(self) -> bool: ...
    def enter_task(self, task: _ZoneTaskContext) -> None: ...
    def exit_task(self, task: _ZoneTaskContext) -> None: ...
    def enter_freeze(self, freeze: _ZoneFreezeContext) -> None: ...
    def exit_freeze(self, freeze: _ZoneFreezeContext) -> None: ...
    def pause(self) -> None: ...
    def reset(self) -> None: ...

class TimeoutManager:
    _loop: Any
    _zones: Any
    _globals: Any
    _freezes: Any
    def __init__(self) -> None: ...
    @property
    def zones_done(self) -> bool: ...
    @property
    def freezes_done(self) -> bool: ...
    @property
    def zones(self) -> dict[str, _ZoneTimeoutManager]: ...
    @property
    def global_tasks(self) -> list[_GlobalTaskContext]: ...
    @property
    def global_freezes(self) -> list[_GlobalFreezeContext]: ...
    def drop_zone(self, zone_name: str) -> None: ...
    def async_timeout(self, timeout: float, zone_name: str = ..., cool_down: float = ...) -> Union[_ZoneTaskContext, _GlobalTaskContext]: ...
    def async_freeze(self, zone_name: str = ...) -> Union[_ZoneFreezeContext, _GlobalFreezeContext]: ...
    def freeze(self, zone_name: str = ...) -> Union[_ZoneFreezeContext, _GlobalFreezeContext]: ...



import asyncio
import datetime
import enum
import voluptuous as vol
from collections.abc import Awaitable, Coroutine, Iterable, Mapping
from homeassistant import block_async_io as block_async_io, loader as loader, util as util
from homeassistant.auth import AuthManager as AuthManager
from homeassistant.components.http import HomeAssistantHTTP as HomeAssistantHTTP
from homeassistant.config_entries import ConfigEntries as ConfigEntries
from homeassistant.const import ATTR_DOMAIN as ATTR_DOMAIN, ATTR_FRIENDLY_NAME as ATTR_FRIENDLY_NAME, ATTR_NOW as ATTR_NOW, ATTR_SECONDS as ATTR_SECONDS, ATTR_SERVICE as ATTR_SERVICE, ATTR_SERVICE_DATA as ATTR_SERVICE_DATA, CONF_UNIT_SYSTEM_IMPERIAL as CONF_UNIT_SYSTEM_IMPERIAL, EVENT_CALL_SERVICE as EVENT_CALL_SERVICE, EVENT_CORE_CONFIG_UPDATE as EVENT_CORE_CONFIG_UPDATE, EVENT_HOMEASSISTANT_CLOSE as EVENT_HOMEASSISTANT_CLOSE, EVENT_HOMEASSISTANT_FINAL_WRITE as EVENT_HOMEASSISTANT_FINAL_WRITE, EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START, EVENT_HOMEASSISTANT_STARTED as EVENT_HOMEASSISTANT_STARTED, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, EVENT_SERVICE_REGISTERED as EVENT_SERVICE_REGISTERED, EVENT_SERVICE_REMOVED as EVENT_SERVICE_REMOVED, EVENT_STATE_CHANGED as EVENT_STATE_CHANGED, EVENT_TIMER_OUT_OF_SYNC as EVENT_TIMER_OUT_OF_SYNC, EVENT_TIME_CHANGED as EVENT_TIME_CHANGED, LENGTH_METERS as LENGTH_METERS, MATCH_ALL as MATCH_ALL, MAX_LENGTH_EVENT_EVENT_TYPE as MAX_LENGTH_EVENT_EVENT_TYPE, MAX_LENGTH_STATE_STATE as MAX_LENGTH_STATE_STATE, __version__ as __version__
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError, InvalidEntityFormatError as InvalidEntityFormatError, InvalidStateError as InvalidStateError, MaxLengthExceeded as MaxLengthExceeded, ServiceNotFound as ServiceNotFound, Unauthorized as Unauthorized
from homeassistant.util import location as location
from homeassistant.util.async_ import fire_coroutine_threadsafe as fire_coroutine_threadsafe, run_callback_threadsafe as run_callback_threadsafe, shutdown_run_callback_threadsafe as shutdown_run_callback_threadsafe
from homeassistant.util.timeout import TimeoutManager as TimeoutManager
from homeassistant.util.unit_system import IMPERIAL_SYSTEM as IMPERIAL_SYSTEM, METRIC_SYSTEM as METRIC_SYSTEM, UnitSystem as UnitSystem
from typing import Any, Callable, TypeVar

STAGE_1_SHUTDOWN_TIMEOUT: int
STAGE_2_SHUTDOWN_TIMEOUT: int
STAGE_3_SHUTDOWN_TIMEOUT: int
T = TypeVar('T')
_UNDEF: dict
CALLABLE_T = TypeVar('CALLABLE_T', bound=Callable)
CALLBACK_TYPE = Callable[[], None]
CORE_STORAGE_KEY: str
CORE_STORAGE_VERSION: int
DOMAIN: str
BLOCK_LOG_TIMEOUT: int
SERVICE_CALL_LIMIT: int
SOURCE_DISCOVERED: str
SOURCE_STORAGE: str
SOURCE_YAML: str
TIMEOUT_EVENT_START: int
_LOGGER: Any

def split_entity_id(entity_id: str) -> list[str]: ...

VALID_ENTITY_ID: Any

def valid_entity_id(entity_id: str) -> bool: ...
def valid_state(state: str) -> bool: ...
def callback(func: CALLABLE_T) -> CALLABLE_T: ...
def is_callback(func: Callable[..., Any]) -> bool: ...

class HassJobType(enum.Enum):
    Coroutinefunction: int
    Callback: int
    Executor: int

class HassJob:
    __slots__: Any
    target: Any
    job_type: Any
    def __init__(self, target: Callable) -> None: ...
    def __repr__(self) -> str: ...

def _get_callable_job_type(target: Callable) -> HassJobType: ...

class CoreState(enum.Enum):
    not_running: str
    starting: str
    running: str
    stopping: str
    final_write: str
    stopped: str
    def __str__(self) -> str: ...

class HomeAssistant:
    auth: AuthManager
    http: HomeAssistantHTTP
    config_entries: ConfigEntries
    loop: Any
    _pending_tasks: Any
    _track_task: bool
    bus: Any
    services: Any
    states: Any
    config: Any
    components: Any
    helpers: Any
    data: Any
    state: Any
    exit_code: int
    _stopped: Any
    timeout: Any
    def __init__(self) -> None: ...
    @property
    def is_running(self) -> bool: ...
    @property
    def is_stopping(self) -> bool: ...
    def start(self) -> int: ...
    async def async_run(self, *, attach_signals: bool = ...) -> int: ...
    async def async_start(self) -> None: ...
    def add_job(self, target: Callable[..., Any], *args: Any) -> None: ...
    def async_add_job(self, target: Callable[..., Any], *args: Any) -> Union[asyncio.Future, None]: ...
    def async_add_hass_job(self, hassjob: HassJob, *args: Any) -> Union[asyncio.Future, None]: ...
    def create_task(self, target: Awaitable) -> None: ...
    def async_create_task(self, target: Awaitable) -> asyncio.Task: ...
    def async_add_executor_job(self, target: Callable[..., T], *args: Any) -> Awaitable[T]: ...
    def async_track_tasks(self) -> None: ...
    def async_stop_track_tasks(self) -> None: ...
    def async_run_hass_job(self, hassjob: HassJob, *args: Any) -> Union[asyncio.Future, None]: ...
    def async_run_job(self, target: Callable[..., Union[None, Awaitable]], *args: Any) -> Union[asyncio.Future, None]: ...
    def block_till_done(self) -> None: ...
    async def async_block_till_done(self) -> None: ...
    async def _await_and_log_pending(self, pending: Iterable[Awaitable[Any]]) -> None: ...
    def stop(self) -> None: ...
    async def async_stop(self, exit_code: int = ..., *, force: bool = ...) -> None: ...

class Context:
    user_id: str
    parent_id: Union[str, None]
    id: str
    def as_dict(self) -> dict[str, Union[str, None]]: ...
    def __init__(self, user_id, parent_id, id) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...

class EventOrigin(enum.Enum):
    local: str
    remote: str
    def __str__(self) -> str: ...

class Event:
    __slots__: Any
    event_type: Any
    data: Any
    origin: Any
    time_fired: Any
    context: Any
    def __init__(self, event_type: str, data: Union[dict[str, Any], None] = ..., origin: EventOrigin = ..., time_fired: Union[datetime.datetime, None] = ..., context: Union[Context, None] = ...) -> None: ...
    def __hash__(self) -> int: ...
    def as_dict(self) -> dict[str, Any]: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...

class EventBus:
    _listeners: Any
    _hass: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_listeners(self) -> dict[str, int]: ...
    @property
    def listeners(self) -> dict[str, int]: ...
    def fire(self, event_type: str, event_data: Union[dict, None] = ..., origin: EventOrigin = ..., context: Union[Context, None] = ...) -> None: ...
    def async_fire(self, event_type: str, event_data: Union[dict[str, Any], None] = ..., origin: EventOrigin = ..., context: Union[Context, None] = ..., time_fired: Union[datetime.datetime, None] = ...) -> None: ...
    def listen(self, event_type: str, listener: Callable) -> CALLBACK_TYPE: ...
    def async_listen(self, event_type: str, listener: Callable, event_filter: Union[Callable, None] = ...) -> CALLBACK_TYPE: ...
    def _async_listen_filterable_job(self, event_type: str, filterable_job: tuple[HassJob, Union[Callable, None]]) -> CALLBACK_TYPE: ...
    def listen_once(self, event_type: str, listener: Callable[[Event], None]) -> CALLBACK_TYPE: ...
    def async_listen_once(self, event_type: str, listener: Callable) -> CALLBACK_TYPE: ...
    def _async_remove_listener(self, event_type: str, filterable_job: tuple[HassJob, Union[Callable, None]]) -> None: ...

class State:
    __slots__: Any
    entity_id: Any
    state: Any
    attributes: Any
    last_updated: Any
    last_changed: Any
    context: Any
    _as_dict: Any
    def __init__(self, entity_id: str, state: str, attributes: Union[Mapping[str, Any], None] = ..., last_changed: Union[datetime.datetime, None] = ..., last_updated: Union[datetime.datetime, None] = ..., context: Union[Context, None] = ..., validate_entity_id: Union[bool, None] = ...) -> None: ...
    @property
    def name(self) -> str: ...
    def as_dict(self) -> dict: ...
    @classmethod
    def from_dict(cls, json_dict: dict) -> Any: ...
    def __eq__(self, other: Any) -> bool: ...
    def __repr__(self) -> str: ...

class StateMachine:
    _states: Any
    _reservations: Any
    _bus: Any
    _loop: Any
    def __init__(self, bus: EventBus, loop: asyncio.events.AbstractEventLoop) -> None: ...
    def entity_ids(self, domain_filter: Union[str, None] = ...) -> list[str]: ...
    def async_entity_ids(self, domain_filter: Union[str, Iterable, None] = ...) -> list[str]: ...
    def async_entity_ids_count(self, domain_filter: Union[str, Iterable, None] = ...) -> int: ...
    def all(self, domain_filter: Union[str, Iterable, None] = ...) -> list[State]: ...
    def async_all(self, domain_filter: Union[str, Iterable, None] = ...) -> list[State]: ...
    def get(self, entity_id: str) -> Union[State, None]: ...
    def is_state(self, entity_id: str, state: str) -> bool: ...
    def remove(self, entity_id: str) -> bool: ...
    def async_remove(self, entity_id: str, context: Union[Context, None] = ...) -> bool: ...
    def set(self, entity_id: str, new_state: str, attributes: Union[Mapping[str, Any], None] = ..., force_update: bool = ..., context: Union[Context, None] = ...) -> None: ...
    def async_reserve(self, entity_id: str) -> None: ...
    def async_available(self, entity_id: str) -> bool: ...
    def async_set(self, entity_id: str, new_state: str, attributes: Union[Mapping[str, Any], None] = ..., force_update: bool = ..., context: Union[Context, None] = ...) -> None: ...

class Service:
    __slots__: Any
    job: Any
    schema: Any
    def __init__(self, func: Callable, schema: Union[vol.Schema, None], context: Union[Context, None] = ...) -> None: ...

class ServiceCall:
    __slots__: Any
    domain: Any
    service: Any
    data: Any
    context: Any
    def __init__(self, domain: str, service: str, data: Union[dict, None] = ..., context: Union[Context, None] = ...) -> None: ...
    def __repr__(self) -> str: ...

class ServiceRegistry:
    _services: Any
    _hass: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    @property
    def services(self) -> dict[str, dict[str, Service]]: ...
    def async_services(self) -> dict[str, dict[str, Service]]: ...
    def has_service(self, domain: str, service: str) -> bool: ...
    def register(self, domain: str, service: str, service_func: Callable, schema: Union[vol.Schema, None] = ...) -> None: ...
    def async_register(self, domain: str, service: str, service_func: Callable, schema: Union[vol.Schema, None] = ...) -> None: ...
    def remove(self, domain: str, service: str) -> None: ...
    def async_remove(self, domain: str, service: str) -> None: ...
    def call(self, domain: str, service: str, service_data: Union[dict, None] = ..., blocking: bool = ..., context: Union[Context, None] = ..., limit: Union[float, None] = ..., target: Union[dict, None] = ...) -> Union[bool, None]: ...
    async def async_call(self, domain: str, service: str, service_data: Union[dict, None] = ..., blocking: bool = ..., context: Union[Context, None] = ..., limit: Union[float, None] = ..., target: Union[dict, None] = ...) -> Union[bool, None]: ...
    def _run_service_in_background(self, coro_or_task: Union[Coroutine, asyncio.Task], service_call: ServiceCall) -> None: ...
    async def _execute_service(self, handler: Service, service_call: ServiceCall) -> None: ...

class Config:
    hass: Any
    latitude: int
    longitude: int
    elevation: int
    location_name: str
    time_zone: str
    units: Any
    internal_url: Any
    external_url: Any
    currency: str
    config_source: str
    skip_pip: bool
    components: Any
    api: Any
    config_dir: Any
    allowlist_external_dirs: Any
    allowlist_external_urls: Any
    media_dirs: Any
    safe_mode: bool
    legacy_templates: bool
    def __init__(self, hass: HomeAssistant) -> None: ...
    def distance(self, lat: float, lon: float) -> Union[float, None]: ...
    def path(self, *path: str) -> str: ...
    def is_allowed_external_url(self, url: str) -> bool: ...
    def is_allowed_path(self, path: str) -> bool: ...
    def as_dict(self) -> dict: ...
    def set_time_zone(self, time_zone_str: str) -> None: ...
    def _update(self, source: str, *, latitude: Union[float, None] = ..., longitude: Union[float, None] = ..., elevation: Union[int, None] = ..., unit_system: Union[str, None] = ..., location_name: Union[str, None] = ..., time_zone: Union[str, None] = ..., external_url: Union[str, dict, None] = ..., internal_url: Union[str, dict, None] = ..., currency: Union[str, None] = ...) -> None: ...
    async def async_update(self, **kwargs: Any) -> None: ...
    async def async_load(self) -> None: ...
    async def async_store(self) -> None: ...

def _async_create_timer(hass: HomeAssistant) -> None: ...



import aiohttp
from aiohttp import web
from collections.abc import Awaitable
from homeassistant import config_entries as config_entries
from homeassistant.const import EVENT_HOMEASSISTANT_CLOSE as EVENT_HOMEASSISTANT_CLOSE, __version__ as __version__
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.frame import warn_use as warn_use
from homeassistant.loader import bind_hass as bind_hass
from typing import Any, Callable

DATA_CONNECTOR: str
DATA_CONNECTOR_NOTVERIFY: str
DATA_CLIENTSESSION: str
DATA_CLIENTSESSION_NOTVERIFY: str
SERVER_SOFTWARE: Any
WARN_CLOSE_MSG: str

def async_get_clientsession(hass: HomeAssistant, verify_ssl: bool = ...) -> aiohttp.ClientSession: ...
def async_create_clientsession(hass: HomeAssistant, verify_ssl: bool = ..., auto_cleanup: bool = ..., **kwargs: Any) -> aiohttp.ClientSession: ...
def _async_create_clientsession(hass: HomeAssistant, verify_ssl: bool = ..., auto_cleanup_method: Union[Callable[[HomeAssistant, aiohttp.ClientSession], None], None] = ..., **kwargs: Any) -> aiohttp.ClientSession: ...
async def async_aiohttp_proxy_web(hass: HomeAssistant, request: web.BaseRequest, web_coro: Awaitable[aiohttp.ClientResponse], buffer_size: int = ..., timeout: int = ...) -> Union[web.StreamResponse, None]: ...
async def async_aiohttp_proxy_stream(hass: HomeAssistant, request: web.BaseRequest, stream: aiohttp.StreamReader, content_type: Union[str, None], buffer_size: int = ..., timeout: int = ...) -> web.StreamResponse: ...
def _async_register_clientsession_shutdown(hass: HomeAssistant, clientsession: aiohttp.ClientSession) -> None: ...
def _async_register_default_clientsession_shutdown(hass: HomeAssistant, clientsession: aiohttp.ClientSession) -> None: ...
def _async_get_connector(hass: HomeAssistant, verify_ssl: bool = ...) -> aiohttp.BaseConnector: ...



from homeassistant.components.sensor import SensorEntityDescription as SensorEntityDescription

class AccuWeatherSensorDescription(SensorEntityDescription):
    unit_metric: Union[str, None]
    unit_imperial: Union[str, None]



from . import AUTH_PROVIDERS as AUTH_PROVIDERS, AUTH_PROVIDER_SCHEMA as AUTH_PROVIDER_SCHEMA, AuthProvider as AuthProvider, LoginFlow as LoginFlow
from ..models import Credentials as Credentials, UserMeta as UserMeta
from collections.abc import Mapping
from homeassistant.const import CONF_ID as CONF_ID
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from typing import Any

STORAGE_VERSION: int
STORAGE_KEY: str

def _disallow_id(conf: dict[str, Any]) -> dict[str, Any]: ...

CONFIG_SCHEMA: Any

def async_get_provider(hass: HomeAssistant) -> HassAuthProvider: ...

class InvalidAuth(HomeAssistantError): ...
class InvalidUser(HomeAssistantError): ...

class Data:
    hass: Any
    _store: Any
    _data: Any
    is_legacy: bool
    def __init__(self, hass: HomeAssistant) -> None: ...
    def normalize_username(self, username: str) -> str: ...
    async def async_load(self) -> None: ...
    @property
    def users(self) -> list[dict[str, str]]: ...
    def validate_login(self, username: str, password: str) -> None: ...
    def hash_password(self, password: str, for_storage: bool = ...) -> bytes: ...
    def add_auth(self, username: str, password: str) -> None: ...
    def async_remove_auth(self, username: str) -> None: ...
    def change_password(self, username: str, new_password: str) -> None: ...
    async def async_save(self) -> None: ...

class HassAuthProvider(AuthProvider):
    DEFAULT_TITLE: str
    data: Any
    _init_lock: Any
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    async def async_initialize(self) -> None: ...
    async def async_login_flow(self, context: Union[dict, None]) -> LoginFlow: ...
    async def async_validate_login(self, username: str, password: str) -> None: ...
    async def async_add_auth(self, username: str, password: str) -> None: ...
    async def async_remove_auth(self, username: str) -> None: ...
    async def async_change_password(self, username: str, new_password: str) -> None: ...
    async def async_get_or_create_credentials(self, flow_result: Mapping[str, str]) -> Credentials: ...
    async def async_user_meta_for_credentials(self, credentials: Credentials) -> UserMeta: ...
    async def async_will_remove_credentials(self, credentials: Credentials) -> None: ...

class HassLoginFlow(LoginFlow):
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...



from . import DOMAIN as DOMAIN, DomainData as DomainData
from aioesphomeapi import DeviceInfo as DeviceInfo
from homeassistant.components import zeroconf as zeroconf
from homeassistant.config_entries import ConfigFlow as ConfigFlow
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_NAME as CONF_NAME, CONF_PASSWORD as CONF_PASSWORD, CONF_PORT as CONF_PORT
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from typing import Any

class EsphomeFlowHandler(ConfigFlow):
    VERSION: int
    _host: Any
    _port: Any
    _password: Any
    def __init__(self) -> None: ...
    async def _async_step_user_base(self, user_input: Union[dict[str, Any], None] = ..., error: Union[str, None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    @property
    def _name(self) -> Union[str, None]: ...
    @_name.setter
    def _name(self, value: str) -> None: ...
    def _set_user_input(self, user_input: Union[dict[str, Any], None]) -> None: ...
    async def _async_authenticate_or_add(self, user_input: Union[dict[str, Any], None]) -> FlowResult: ...
    async def async_step_discovery_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    def _async_get_entry(self) -> FlowResult: ...
    async def async_step_authenticate(self, user_input: Union[dict[str, Any], None] = ..., error: Union[str, None] = ...) -> FlowResult: ...
    async def fetch_device_info(self) -> tuple[Union[str, None], Union[DeviceInfo, None]]: ...
    async def try_login(self) -> Union[str, None]: ...



from .const import POLICY_CONTROL as POLICY_CONTROL, POLICY_EDIT as POLICY_EDIT, POLICY_READ as POLICY_READ, SUBCAT_ALL as SUBCAT_ALL
from .models import PermissionLookup as PermissionLookup
from .types import CategoryType as CategoryType, SubCategoryDict as SubCategoryDict, ValueType as ValueType
from .util import SubCatLookupType as SubCatLookupType, compile_policy as compile_policy, lookup_all as lookup_all
from typing import Any, Callable

SINGLE_ENTITY_SCHEMA: Any
ENTITY_DOMAINS: str
ENTITY_AREAS: str
ENTITY_DEVICE_IDS: str
ENTITY_ENTITY_IDS: str
ENTITY_VALUES_SCHEMA: Any
ENTITY_POLICY_SCHEMA: Any

def _lookup_domain(perm_lookup: PermissionLookup, domains_dict: SubCategoryDict, entity_id: str) -> Union[ValueType, None]: ...
def _lookup_area(perm_lookup: PermissionLookup, area_dict: SubCategoryDict, entity_id: str) -> Union[ValueType, None]: ...
def _lookup_device(perm_lookup: PermissionLookup, devices_dict: SubCategoryDict, entity_id: str) -> Union[ValueType, None]: ...
def _lookup_entity_id(perm_lookup: PermissionLookup, entities_dict: SubCategoryDict, entity_id: str) -> Union[ValueType, None]: ...
def compile_entities(policy: CategoryType, perm_lookup: PermissionLookup) -> Callable[[str, str], bool]: ...



import abc
from abc import abstractmethod
from collections.abc import Iterable
from contextvars import ContextVar
from homeassistant.const import ATTR_ASSUMED_STATE as ATTR_ASSUMED_STATE, ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_ICON as ATTR_ICON, ATTR_NAME as ATTR_NAME, CONF_ENTITIES as CONF_ENTITIES, CONF_ICON as CONF_ICON, CONF_NAME as CONF_NAME, ENTITY_MATCH_ALL as ENTITY_MATCH_ALL, ENTITY_MATCH_NONE as ENTITY_MATCH_NONE, EVENT_HOMEASSISTANT_START as EVENT_HOMEASSISTANT_START, SERVICE_RELOAD as SERVICE_RELOAD, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from homeassistant.core import CoreState as CoreState, HomeAssistant as HomeAssistant, callback as callback, split_entity_id as split_entity_id
from homeassistant.helpers.entity import Entity as Entity, async_generate_entity_id as async_generate_entity_id
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from homeassistant.helpers.integration_platform import async_process_integration_platforms as async_process_integration_platforms
from homeassistant.helpers.reload import async_reload_integration_platforms as async_reload_integration_platforms
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

DOMAIN: str
GROUP_ORDER: str
ENTITY_ID_FORMAT: Any
CONF_ALL: str
ATTR_ADD_ENTITIES: str
ATTR_AUTO: str
ATTR_ENTITIES: str
ATTR_OBJECT_ID: str
ATTR_ORDER: str
ATTR_ALL: str
SERVICE_SET: str
SERVICE_REMOVE: str
PLATFORMS: Any
REG_KEY: Any
_LOGGER: Any
current_domain: ContextVar[str]

def _conf_preprocess(value): ...

GROUP_SCHEMA: Any
CONFIG_SCHEMA: Any

class GroupIntegrationRegistry:
    on_off_mapping: dict[str, str]
    off_on_mapping: dict[str, str]
    on_states_by_domain: dict[str, set]
    exclude_domains: set
    def exclude_domain(self) -> None: ...
    def on_off_states(self, on_states: set, off_state: str) -> None: ...

def is_on(hass, entity_id): ...
def expand_entity_ids(hass: HomeAssistant, entity_ids: Iterable[Any]) -> list[str]: ...
def get_entity_ids(hass: HomeAssistant, entity_id: str, domain_filter: Union[str, None] = ...) -> list[str]: ...
def groups_with_entity(hass: HomeAssistant, entity_id: str) -> list[str]: ...
async def async_setup(hass, config): ...
async def _process_group_platform(hass, domain, platform) -> None: ...
async def _async_process_config(hass, config, component) -> None: ...

class GroupEntity(Entity, metaclass=abc.ABCMeta):
    @property
    def should_poll(self) -> bool: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_defer_or_update_ha_state(self) -> None: ...
    @abstractmethod
    async def async_update(self) -> None: ...

class Group(Entity):
    hass: Any
    _name: Any
    _state: Any
    _icon: Any
    _on_off: Any
    _assumed: Any
    _on_states: Any
    user_defined: Any
    mode: Any
    _order: Any
    _assumed_state: bool
    _async_unsub_state_changed: Any
    def __init__(self, hass, name, order: Any | None = ..., icon: Any | None = ..., user_defined: bool = ..., entity_ids: Any | None = ..., mode: Any | None = ...) -> None: ...
    @staticmethod
    def create_group(hass, name, entity_ids: Any | None = ..., user_defined: bool = ..., icon: Any | None = ..., object_id: Any | None = ..., mode: Any | None = ..., order: Any | None = ...): ...
    @staticmethod
    async def async_create_group(hass, name, entity_ids: Any | None = ..., user_defined: bool = ..., icon: Any | None = ..., object_id: Any | None = ..., mode: Any | None = ..., order: Any | None = ...): ...
    @property
    def should_poll(self): ...
    @property
    def name(self): ...
    @name.setter
    def name(self, value) -> None: ...
    @property
    def state(self): ...
    @property
    def icon(self): ...
    @icon.setter
    def icon(self, value) -> None: ...
    @property
    def extra_state_attributes(self): ...
    @property
    def assumed_state(self): ...
    def update_tracked_entity_ids(self, entity_ids) -> None: ...
    async def async_update_tracked_entity_ids(self, entity_ids) -> None: ...
    tracking: Any
    trackable: Any
    def _set_tracked(self, entity_ids) -> None: ...
    def _async_start(self, *_) -> None: ...
    def _async_start_tracking(self) -> None: ...
    def _async_stop(self) -> None: ...
    async def async_update(self) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    async def _async_state_changed_listener(self, event) -> None: ...
    def _reset_tracked_state(self) -> None: ...
    def _see_state(self, new_state) -> None: ...
    def _async_update_group_state(self, tr_state: Any | None = ...) -> None: ...



from enum import Enum
from homeassistant.components.climate.const import CURRENT_HVAC_COOL as CURRENT_HVAC_COOL, CURRENT_HVAC_DRY as CURRENT_HVAC_DRY, CURRENT_HVAC_FAN as CURRENT_HVAC_FAN, CURRENT_HVAC_HEAT as CURRENT_HVAC_HEAT, CURRENT_HVAC_OFF as CURRENT_HVAC_OFF, HVAC_MODE_AUTO as HVAC_MODE_AUTO, HVAC_MODE_COOL as HVAC_MODE_COOL, HVAC_MODE_DRY as HVAC_MODE_DRY, HVAC_MODE_FAN_ONLY as HVAC_MODE_FAN_ONLY, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, PRESET_COMFORT as PRESET_COMFORT, PRESET_ECO as PRESET_ECO, PRESET_NONE as PRESET_NONE, PRESET_SLEEP as PRESET_SLEEP
from typing import Any, Final

DOMAIN: Final[str]
KNX_ADDRESS: Final[str]
CONF_INVERT: Final[str]
CONF_KNX_EXPOSE: Final[str]
CONF_KNX_INDIVIDUAL_ADDRESS: Final[str]
CONF_KNX_ROUTING: Final[str]
CONF_KNX_TUNNELING: Final[str]
CONF_RESET_AFTER: Final[str]
CONF_RESPOND_TO_READ: Final[str]
CONF_STATE_ADDRESS: Final[str]
CONF_SYNC_STATE: Final[str]
ATTR_COUNTER: Final[str]
ATTR_LAST_KNX_UPDATE: Final[str]
ATTR_SOURCE: Final[str]

class ColorTempModes(Enum):
    ABSOLUTE: str
    RELATIVE: str

class SupportedPlatforms(Enum):
    BINARY_SENSOR: str
    CLIMATE: str
    COVER: str
    FAN: str
    LIGHT: str
    NOTIFY: str
    NUMBER: str
    SCENE: str
    SELECT: str
    SENSOR: str
    SWITCH: str
    WEATHER: str

CONTROLLER_MODES: Final[Any]
CURRENT_HVAC_ACTIONS: Final[Any]
PRESET_MODES: Final[Any]
